__author__ = 'Philipp Konrad'

import re
import unittest
import string
import itertools as it

from lxml import etree
from lxml.builder import ElementMaker


class CmlBuilder(object):
    '''
    CmlBuilder helps you to create a Crowdflower CML file.
    The following elements are included:

        * text
        * textarea
        * checkbox
        * checkboxes
        * radio buttons
        * dropdown menu
        * rating

    Each of these tags be configured with several attributes. For an
    exact listening please consult each method.

    CmlBuilder support fluid programming techniques. Of course,
    each method can be applied separately to the builder.

    Usage

        >>> builder = CmlBuilder()
        >>> builder.text('Your age', name='ageid', validates='required positiveInteger')\
        ...        .dropdown_menu('Gender', ['male', 'female', 'supercow'], 'gender', default='supercow',
        ...                      only_if='ageid')\
        ...        .textarea('Essay', name='essay', default='Write about your essay here.', validates='minLength:100')\
        ...        .radio_buttons('Essay Topic', ['romance', 'thriller', 'crime'], name='topic')\
        ...        .radio_buttons_rating('How did you like this exercise (10 = Great, 1 = Not at all)', points=10,
        ...                              name='rating')
    '''

    _maker = ElementMaker(namespace='cml', nsmap={'cml':'cml'})
    _ATTR_VALIDATES_FIELDS = ('required',
                                'url',
                                'integer',
                                'positiveInteger',
                                'numeric',
                                'alpha',
                                'alphanum',
                                'date')

    _ATTR_VALIDATES_PARAM_FIELDS = ('integerRange',
                                   'minLength',
                                   'rangeLength')

    _INTEGER_RANGE_PATTERN = r'''integerRange:{min:-?\d+,max:-?\d+}'''
    _MIN_LENGTH_PATTERN = r'''minLength:-?\d+'''
    _RANGE_LENGTH_PATTERN = r'''rangeLength:{min:-?\d+,max:-?\d+}'''


    def __init__(self):
        self._root = self._maker.root()
        self._attribute_checks = {'name' : self._attr_name_is_ok,
                                  'points' : self._attr_point_is_ok,
                                  'validates':self._attr_validates_is_ok}


    def _create_base_element(self, element_type, label='', name='', value='',
                            default='', instructions='', only_if='', validates='',
                            aggregation='', points=0):
        '''

        '''

        # get all local variables
        kwargs = locals()

        is_param = lambda k: k not in ['element_type', 'cls', 'self']

        # XML documents only can contain strings therefore unicode(v)
        # moreover, we filter all non-parameters and self, cls.
        filtered_kwargs = {k:unicode(v) for k, v in kwargs.iteritems() if is_param(k) and v}

        cleaned_kwargs = {}
        error_msg = 'The attribute is invalid: %s:%s'

        # check the sanity of all attributes.
        # each check function is looked up in a _attribute_checks dictionary
        # in the case of a bad attribute an exception is raised.
        for attr, v in filtered_kwargs.iteritems():
            check_func = self._attribute_checks.get(attr)
            if check_func: assert check_func(attr, v), error_msg % (attr, v)
            cleaned_kwargs[attr] = v

        elm_properties = {'elm_type':element_type,
                          'kwargs':cleaned_kwargs}

        # dynamically build the XML element with element builder and eval
        cmd = 'self._maker.%(elm_type)s(**%(kwargs)s)'% elm_properties
        elm = eval(cmd)

        return elm


    def _attr_name_is_ok(self, key='', value=''):
        '''
        Name attributes must be:

            * only contain lower case ASCII letters and digits
            * no other symbols like points or underlines
            * no spaces
        '''

        if not value: return False
        valid_chars = string.ascii_lowercase + string.digits
        is_valid_char = lambda c: c in valid_chars
        return all(it.imap(is_valid_char, value))


    def _attr_point_is_ok(self, key, value):
        '''
        There need to be at least 2 ratings points
        for the user.
        '''
        return value > 1

    def _attr_validates_is_ok(self, key, value):
        '''
        Check all tag attributes for correctness.
        We differ between:

            * simple attributes like `required` or `url`
            * paramatrized attributes like `minLength:10`.

        '''

        def _in(attr_value):

            # is the validator a common simple validator like 'required'
            if attr_value in self._ATTR_VALIDATES_FIELDS: return True

            # does the validator accept parameters like: minLength:4
            is_parametrized_validator =  any(attr_value.startswith(fld)
                                             for fld in self._ATTR_VALIDATES_PARAM_FIELDS)

            if not is_parametrized_validator: return False

            # check the correctness of the paramatrized validator.
            is_integer_range = (attr_value.startswith('integerRange') and
                                re.match(self._INTEGER_RANGE_PATTERN, attr_value))
            is_min_length = (attr_value.startswith('minLength') and
                             re.match(self._MIN_LENGTH_PATTERN, attr_value))
            is_range_length = (attr_value.startswith('rangeLength') and
                               re.match(self._RANGE_LENGTH_PATTERN, attr_value))

            if is_integer_range or is_min_length or is_range_length:
                return True

            return False


        assert value, 'No value for validates attribute provided!'

        values = value.split(' ')
        error_msg = 'You only can assign two values to the validates attribute: %s'
        assert len(values) < 3, error_msg % values

        if not all(_in(val) for val in values): return False

        return True


    def text(self, label='', name='', default='', instructions='', only_if='',
                   validates=''):
        '''
        Create one liner text tag. You can find an example on the crowdflower website:
        http://success.crowdflower.com/customer/portal/articles/1290342-cml-crowdflower-markup-language#CMLtext

        :param str label:  This will be displayed next to the generated form element.
                           If no name attribute is specified on a base tag, this
                           will be converted into a name by removing alpha-numeric
                           characters and replacing spaces with an underscore.

        :param str instructions: Every CML base tag can have an instructions attribute.
                                 This will be displayed next to the generated form
                                 element to help clarify the desired input.
                                 If both an instructions attribute and a
                                 `<cml:instructions />` tag is specified,
                                 only the value of the attribute will be used.

        :param str name: This must be unique across all base tags. The name should not
                         contain capital letters, spaces, or non alpha-numeric characters.
                         This will become a column header in your generated CSV.

        :param str default: this provides an example input for the user
                            and disappears once the user selects the form element.
                            Default values will not be submitted.

        :param str only_if: Every CML base tag can have an `only_if` attribute.
                            The value of this attribute should be the name of the
                            field a user must complete before this field or
                            group of fields will be displayed.

        :param str validates: CML supports a number of pre-made validation methods
                              to ensure data integrity. Some validators normalize
                              the input, making it possible to create gold for
                              complex data like phone numbers, addresses, and URLs.

                              * `required`: On a free text input, at least one
                                            non-whitespace character must be present.
                                            On a multiple choice or drop-down field,
                                            it enforces at least one item to be selected.

                              * `integer`

                              * `positiveInteger`

                              * `numeric`: Requires an integer or floating-point value.

                              * `integerRange`: Ensures that the contributor inputs an
                                                integer within a given range. Note:
                                                The values are inclusive: {min:1,max:100}

                              * `alpha`: Requires only letters, e.g., "ABCabc"

                              * `alphanum`: Allows only numbers and letters,
                                            e.g., "ahfd723nd"

                              * `date`: Requires a date in MM/DD/YYYY format,
                                        e.g., "01/21/2010"

                              * `minLength`: Ensures that the user's input is at least a
                                             certain number of characters long e.g.
                                             minLength:4

                              * `rangeLength`: Ensures that the user's input is within a
                                               given length range. Note: The values are
                                               inclusive e.g. rangeLength:{min:5,max:32}

                              * `url`: Requires a valid-looking URL,
                                       e.g. "http://crowdflower.com."
        '''

        txt_elm = self._create_base_element('text', label=label,
                                                    name=name,
                                                    default=default,
                                                    instructions=instructions,
                                                    only_if=only_if,
                                                    validates=validates)
        self._root.append(txt_elm)
        return self


    def textarea(self, label='', name='', default='', instructions='', only_if='',
                       validates=''):
        '''
        Create multiline textarea tag. You can find an example on the crowdflower website:
        http://success.crowdflower.com/customer/portal/articles/1290342-cml-crowdflower-markup-language#CMLtextarea

        :param str label:  This will be displayed next to the generated form element.
                           If no name attribute is specified on a base tag, this
                           will be converted into a name by removing alpha-numeric
                           characters and replacing spaces with an underscore.

        :param str name: This must be unique across all base tags. The name should not
                         contain capital letters, spaces, or non alpha-numeric characters.
                         This will become a column header in your generated CSV.

        :param str default: this provides an example input for the user
                            and disappears once the user selects the form element.
                            Default values will not be submitted.

        :param str instructions: Every CML base tag can have an instructions attribute.
                                 This will be displayed next to the generated form
                                 element to help clarify the desired input.
                                 If both an instructions attribute and a
                                 `<cml:instructions />` tag is specified,
                                 only the value of the attribute will be used.

        :param str only_if: Every CML base tag can have an `only_if` attribute.
                            The value of this attribute should be the name of the
                            field a user must complete before this field or
                            group of fields will be displayed.

        :param str validates: CML supports a number of pre-made validation methods
                              to ensure data integrity. Some validators normalize
                              the input, making it possible to create gold for
                              complex data like phone numbers, addresses, and URLs.

                              * `required`: On a free text input, at least one
                                            non-whitespace character must be present.
                                            On a multiple choice or drop-down field,
                                            it enforces at least one item to be selected.

                              * `integer`

                              * `positiveInteger`

                              * `numeric`: Requires an integer or floating-point value.

                              * `integerRange`: Ensures that the contributor inputs an
                                                integer within a given range. Note:
                                                The values are inclusive: e.g.
                                                integerRange:{min:1,max:100}

                              * `alpha`: Requires only letters, e.g., "ABCabc"

                              * `alphanum`: Allows only numbers and letters,
                                            e.g., "ahfd723nd"

                              * `date`: Requires a date in MM/DD/YYYY format,
                                        e.g., "01/21/2010"

                              * `minLength`: Ensures that the user's input is at least a
                                             certain number of characters long e.g.
                                             minLength:4

                              * `rangeLength`: Ensures that the user's input is within a
                                               given length range. Note: The values are
                                               inclusive e.g. rangeLength:{min:5,max:32}

                              * `url`: Requires a valid-looking URL,
                                       e.g. "http://crowdflower.com."
        '''

        txtarea = self._create_base_element('text',
                                            label=label,
                                            name=name,
                                            default=default,
                                            instructions=instructions,
                                            only_if=only_if,
                                            validates=validates)
        self._root.append(txtarea)
        return self


    def checkbox(self, label='', name='', default='', instructions='', only_if='',
                   validates=''):
        '''
        Create a checkbox tag. You can find an example on the crowdflower website:
        http://success.crowdflower.com/customer/portal/articles/1290342-cml-crowdflower-markup-language#CMLcheckbox

        :param str label:  This will be displayed next to the generated form element.
                           If no name attribute is specified on a base tag, this
                           will be converted into a name by removing alpha-numeric
                           characters and replacing spaces with an underscore.

        :param str instructions: Every CML base tag can have an instructions attribute.
                                 This will be displayed next to the generated form
                                 element to help clarify the desired input.
                                 If both an instructions attribute and a
                                 `<cml:instructions />` tag is specified,
                                 only the value of the attribute will be used.

        :param str name: This must be unique across all base tags. The name should not
                         contain capital letters, spaces, or non alpha-numeric characters.
                         This will become a column header in your generated CSV.

        :param str default: this provides an example input for the user
                            and disappears once the user selects the form element.
                            Default values will not be submitted.

        :param str only_if: Every CML base tag can have an `only_if` attribute.
                            The value of this attribute should be the name of the
                            field a user must complete before this field or
                            group of fields will be displayed.

        :param str validates: CML supports a number of pre-made validation methods
                              to ensure data integrity. Some validators normalize
                              the input, making it possible to create gold for
                              complex data like phone numbers, addresses, and URLs.

                              * `required`: On a free text input, at least one
                                            non-whitespace character must be present.
                                            On a multiple choice or drop-down field,
                                            it enforces at least one item to be selected.
        '''


        checkbox = self._create_base_element('checkbox',
                                            label=label,
                                            name=name,
                                            default=default,
                                            instructions=instructions,
                                            only_if=only_if,
                                            validates=validates)
        self._root.append(checkbox)
        return self


    def radio_buttons(self, label, radio_button_strings, name='', instructions='', only_if='',
                            validates=''):
        '''
        Create radio buttons tag. You can find an example on the crowdflower website:
        http://success.crowdflower.com/customer/portal/articles/1290342-cml-crowdflower-markup-language#CMLradios

        Usage:
            >>> builder = CmlBuilder()
            >>> builder.radio_buttons('Please choose a color', ['blue', 'red', 'green'])
            >>> builder.dumps()

        :param str label:  This will be displayed next to the generated form element.
                           If no name attribute is specified on a base tag, this
                           will be converted into a name by removing alpha-numeric
                           characters and replacing spaces with an underscore.

        :param str name: This must be unique across all base tags. The name should not
                         contain capital letters, spaces, or non alpha-numeric characters.
                         This will become a column header in your generated CSV.

        :param str instructions: Every CML base tag can have an instructions attribute.
                                 This will be displayed next to the generated form
                                 element to help clarify the desired input.
                                 If both an instructions attribute and a
                                 `<cml:instructions />` tag is specified,
                                 only the value of the attribute will be used.

        :param str only_if: Every CML base tag can have an `only_if` attribute.
                            The value of this attribute should be the name of the
                            field a user must complete before this field or
                            group of fields will be displayed.

        :param str validates: CML supports a number of pre-made validation methods
                              to ensure data integrity. Some validators normalize
                              the input, making it possible to create gold for
                              complex data like phone numbers, addresses, and URLs.

                              * `required`: On a multiple choice or drop-down field,
                                            it enforces at least one item to be selected.
        '''
        radio_btn_group = self._create_base_element('radios',
                                                    label=label,
                                                    name=name,
                                                    instructions=instructions,
                                                    only_if=only_if,
                                                    validates=validates)

        for radio_btn in radio_button_strings:
            btn = self._create_base_element('radio',
                                            label=radio_btn)
            radio_btn_group.append(btn)

        self._root.append(radio_btn_group)
        return self


    def dropdown_menu(self, label, option_strings, name='', default='', instructions='',
                            only_if='', validates=''):
        '''
        Create a dropdown menu.
        http://success.crowdflower.com/customer/portal/articles/1290342-cml-crowdflower-markup-language-#CMLselect

        Usage:
            >>> builder = CmlBuilder()
            >>> builder.dropdown_menu('Choose a city', ['Vienna', 'Chur', 'Innsbruck'])
            >>> builder.dumps()

        :param str label:  This will be displayed next to the generated form element.
                           If no name attribute is specified on a base tag, this
                           will be converted into a name by removing alpha-numeric
                           characters and replacing spaces with an underscore.

        :param str default: this provides an example input for the user
                            and disappears once the user selects the form element.
                            Default values will not be submitted.

        :param str name: This must be unique across all base tags. The name should not
                         contain capital letters, spaces, or non alpha-numeric characters.
                         This will become a column header in your generated CSV.

        :param str instructions: Every CML base tag can have an instructions attribute.
                                 This will be displayed next to the generated form
                                 element to help clarify the desired input.
                                 If both an instructions attribute and a
                                 `<cml:instructions />` tag is specified,
                                 only the value of the attribute will be used.

        :param str only_if: Every CML base tag can have an `only_if` attribute.
                            The value of this attribute should be the name of the
                            field a user must complete before this field or
                            group of fields will be displayed.

        :param str validates: CML supports a number of pre-made validation methods
                              to ensure data integrity. Some validators normalize
                              the input, making it possible to create gold for
                              complex data like phone numbers, addresses, and URLs.

                              * `required`: On a multiple choice or drop-down field,
                                            it enforces at least one item to be selected.


        '''

        assert option_strings, 'The option_strings parameter is empty!'

        if default:
            error_msg = 'Could not find default value in options: %s %s'
            assert default in option_strings, error_msg % (option_strings, default)

        drpdwn_menu = self._create_base_element('select',
                                                label=label,
                                                name=name,
                                                default=default,
                                                instructions=instructions,
                                                only_if=only_if,
                                                validates=validates)

        for option_str in option_strings:
            option = self._create_base_element('option',
                                               label=option_str)
            drpdwn_menu.append(option)

        self._root.append(drpdwn_menu)
        return self


    def radio_buttons_rating(self, label='', points=5, name='', validates=''):
        '''
        Create a task rating for the Crowdflower users. For an example see this link:
        http://success.crowdflower.com/customer/portal/articles/1290342-cml-crowdflower-markup-language#CMLratings

        Usage:
            >>> builder = CmlBuilder()
            >>> builder.radio_buttons_rating('Please rate the task from 1 to 10', points=10)
            >>> builder.dumps()

        :param str label:  This will be displayed next to the generated form element.
                           If no name attribute is specified on a base tag, this
                           will be converted into a name by removing alpha-numeric
                           characters and replacing spaces with an underscore.

        :param str instructions: Every CML base tag can have an instructions attribute.
                                 This will be displayed next to the generated form
                                 element to help clarify the desired input.
                                 If both an instructions attribute and a
                                 `<cml:instructions />` tag is specified,
                                 only the value of the attribute will be used.

        :param str name: This must be unique across all base tags. The name should not
                         contain capital letters, spaces, or non alpha-numeric characters.
                         This will become a column header in your generated CSV.

        :param str validates: CML supports a number of pre-made validation methods
                              to ensure data integrity. Some validators normalize
                              the input, making it possible to create gold for
                              complex data like phone numbers, addresses, and URLs.

                              * `required`: On a free text input, at least one
                                            non-whitespace character must be present.
                                            On a multiple choice or drop-down field,
                                            it enforces at least one item to be selected.

        http://success.crowdflower.com/customer/portal/articles/1290342-cml-crowdflower-markup-language-#CMLratings
        '''

        ratings = self._create_base_element('ratings',
                                            label=label,
                                            points=points,
                                            name=name,
                                            validates=validates)
        self._root.append(ratings)
        return self

    def checkboxes(self, label, checkbox_strings, name='', default='', instructions='',
                            only_if='', validates=''):
        '''
        Create a group of checkboxes.
        http://success.crowdflower.com/customer/portal/articles/1290342-cml-crowdflower-markup-language-#CMLcheckboxes

        Usage:
            >>> builder = CmlBuilder()
            >>> builder.checkboxes('Which fruits do you like?', ['apples', 'bananas','oranges'])
            >>> builder.dumps()
        '''

        assert checkbox_strings, 'The option_strings parameter is empty!'

        if default:
            error_msg = 'Could not find default value in options: %s %s'
            assert default in checkbox_strings, error_msg % (checkbox_strings, default)

        drpdwn_menu = self._create_base_element('checkboxes',
                                                label=label,
                                                name=name,
                                                default=default,
                                                instructions=instructions,
                                                only_if=only_if,
                                                validates=validates)

        for option_str in checkbox_strings:
            option = self._create_base_element('checkbox',
                                               label=option_str)
            drpdwn_menu.append(option)

        self._root.append(drpdwn_menu)
        return self



    def dumps(self):
        '''
        Return the CML tree as a string.

        Usage:
            >>> builder = CmlBuilder()
            >>> builder.text('Enter your age', name='name_id', validates='required positiveInteger')
            >>> builder.radio_buttons_rating('Rate the exercise', validates='required')
            >>> builder.dumps()
        '''
        children = self._root.getchildren()
        children_str = [etree.tostring(child, pretty_print=True)
                        for child in children if child is not None]
        return ''.join(children_str)


class TestCmlBuilder(unittest.TestCase):


    def test_points(self):

        test_result = '<cml:ratings xmlns:cml="cml" points="5"/>\n'
        builder = CmlBuilder()
        self.assertEquals(builder.radio_buttons_rating().dumps(), test_result)


    def test_text(self):

        builder = CmlBuilder()
        builder.text(label='My test label', name='uniquename')
        self.assertEquals(builder.dumps(),
                          '<cml:text xmlns:cml="cml" name="uniquename" label="My test label"/>\n')


    def test_textarea(self):

        builder = CmlBuilder()
        builder.textarea(label='Some explanation', name='textareaid')
        self.assertEquals(builder.dumps(),
                          '<cml:text xmlns:cml="cml" name="textareaid" label="Some explanation"/>\n')


    def test_dumps(self):

        test_result = '<cml:text xmlns:cml="cml" name="name" label="Sample text field:"/>\n' \
                    + '<cml:text xmlns:cml="cml" label="my textarea"/>\n'

        builder = CmlBuilder()
        builder.text('Sample text field:', name='name')\
               .textarea('my textarea')

        self.assertEquals(builder.dumps(), test_result)


    def test_attr_name_ok(self):

        builder = CmlBuilder()
        self.assertTrue(builder._attr_name_is_ok('key', 'testname'))
        self.assertFalse(builder._attr_name_is_ok('key', 'test_name'))
        self.assertFalse(builder._attr_name_is_ok('key', 'TestName'))
        self.assertFalse(builder._attr_name_is_ok('key', 'test name'))


    def test_attr_validates_ok(self):

        builder = CmlBuilder()

        # basic validation rules

        for config in ['required', 'integer', 'positiveInteger', 'numeric',
                       'integerRange', 'alpha', 'alphanum', 'date', 'minLength',
                       'rangeLength', 'url']:

            self.assertTrue('validates', config)

        # simple invalid values
        for faulty_config in ['.integer', 'wrong', 'URL', 'min_length']:
            self.assertFalse(builder._attr_validates_is_ok('validates', faulty_config))

        # simple composed values
        self.assertTrue(builder._attr_validates_is_ok('validates', 'required url'))
        self.assertFalse(builder._attr_validates_is_ok('validates', 'required, url'))
        self.assertFalse(builder._attr_validates_is_ok('validates', 'date x'))


    def test_attr_validates_ok_complex(self):
        builder = CmlBuilder()
        self.assertTrue(builder._attr_validates_is_ok('validates', 'integerRange:{min:10,max:100}'))
        self.assertTrue(builder._attr_validates_is_ok('validates', 'rangeLength:{min:0,max:100}'))
        self.assertTrue(builder._attr_validates_is_ok('validates', 'minLength:7'))

        self.assertTrue(builder._attr_validates_is_ok('validates', 'integerRange:{min:-4,max:100}'))
        self.assertTrue(builder._attr_validates_is_ok('validates', 'rangeLength:{min:-3000,max:100}'))
        self.assertTrue(builder._attr_validates_is_ok('validates', 'minLength:-10'))


    def test_checkbox(self):

        builder = CmlBuilder()
        builder.checkbox('A single checkbox', name='checkboxcheck')
        self.assertEquals(builder.dumps(),
                          '<cml:checkbox xmlns:cml="cml" name="checkboxcheck" label="A single checkbox"/>\n')


    def test_radioboxes(self):

        builder = CmlBuilder()
        builder.checkboxes('What are cities?', ['Wa', 'Vienna', 'Ba', 'Graz'], name='cities')
        result = '''<cml:checkboxes xmlns:cml="cml" name="cities" label="What are cities?">
  <cml:checkbox label="Wa"/>
  <cml:checkbox label="Vienna"/>
  <cml:checkbox label="Ba"/>
  <cml:checkbox label="Graz"/>
</cml:checkboxes>
'''
        self.assertEquals(builder.dumps(), result)


    def test_dropdown(self):

        builder = CmlBuilder()
        builder.dropdown_menu('Gender', ['male', 'female', 'supercow'], 'gender', default='supercow')

        result = '''<cml:select xmlns:cml="cml" default="supercow" name="gender" label="Gender">
  <cml:option label="male"/>
  <cml:option label="female"/>
  <cml:option label="supercow"/>
</cml:select>
'''
        self.assertEquals(result, builder.dumps())



    def test_cml_creation(self):

        builder = CmlBuilder()
        builder.text('Your age', name='ageid', validates='required positiveInteger')\
               .dropdown_menu('Gender', ['male', 'female', 'supercow'], 'gender', default='supercow',
                              only_if='ageid')\
               .textarea('Essay', name='essay', default='Write about your essay here.', validates='minLength:100')\
               .radio_buttons('Essay Topic', ['romance', 'thriller', 'crime'], name='topic')\
               .radio_buttons_rating('How did you like this exercise (10 = Great, 1 = Not at all', points=10,
                                     name='rating')

        builder.dumps()

if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python

"""
::package eWRT.ws.conceptnet
Access to conceptnet data structures using its REST interface

::author: Albert Weichselbraun <albert.weichselbraun@htwchur.ch>
"""

try:
    from urllib import quote
except ImportError:
    from urllib.parse import quote

from eWRT.ws.conceptnet import Concept, Result, CONCEPTNET_BASE_URL, retrieve_conceptnet_query_result

# maximum number of edges to retrieve
DEFAULT_MAX_EDGE_COUNT = 5000


class LookupResult(Result):
    '''
    An object for handling ConceptNet search results
    '''

    def __init__(self, term=None, rel_type='c', lang='en', pos_tag=None, conceptnet_url=None, strict=False, count=DEFAULT_MAX_EDGE_COUNT):
        '''
        ::param term: the lookup term
        ::param rel_type: the relation type (c).
        ::param lang: the language to use (en).
        ::param pos_tag: an optional lookup pos_tag
        ::param conceptnet_url:
        ::param strict: if set remove all edges that do not contain the
                        exact conceptnet_url
        ::param count: maximum number of edges to retrieve
        ::return: a list of edges matching the lookup query
        '''
        if not conceptnet_url:
            conceptnet_url = "%s/%s/%s/%s" % (CONCEPTNET_BASE_URL,
                                              rel_type, lang, term)
        host, url = conceptnet_url.split("/", 3)[2:]
        conceptnet_url = "http://{host}/{url}?limit={count}".format(
            host=host, url=quote(url).replace('//', '/'), count=count)

        Result.__init__(self, retrieve_conceptnet_query_result(conceptnet_url))

    @staticmethod
    def get_concept_from_url(concept_url):
        '''
        ::param concept_url: the url of the ConceptNet concept
        '''
        _, url_type, lang, concept_name, pos, sense = \
            concept_url.split('/', 6)
        return Concept(lang, concept_name, pos, sense)

    @staticmethod
    def _split(s, delimiter, size):
        '''
        ::param s: the string to split
        ::param delimiter: delimiter
        ::param size: size of the list to return
        '''
        l = s.split(delimiter, size)
        return l + (size - len(l)) * [None]


if __name__ == '__main__':
    r = LookupResult('lord', strict=True)
    from eWRT.ws.conceptnet.util import VALID_SENSE_FILTER, VALID_LANGUAGES
    r.apply_language_filter(VALID_LANGUAGES)
    # r.apply_edge_filter(VALID_SENSE_FILTER)
    #print("\n".join(map(repr, r.get_senses())))
    print(r.get_vsm(stopword_list=()))



#!/usr/bin/env python

""" uses the del.icio.us API to access information about del.icio.us URLs """

# (C)opyrights 2008-2010 by Albert Weichselbraun <albert@weichselbraun.net>
#                           Heinz Peter Lang <hplang@langatium.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__version__ = "$Header$"

import re
import urllib2

from urllib import quote
from time import sleep
from hashlib import md5

from eWRT.access.http import Retrieve
from eWRT.ws.TagInfoService import TagInfoService


class Delicious(TagInfoService):
    """ retrieves data using the del.icio.us API """

    DELICIOUS_SERVICE_URL = "http://del.icio.us/url/%s"
    DELICIOUS_TAG_URL = "http://delicious.com/tag/%s"
    RE_COUNT = re.compile("<p>(\d+) Bookmarks</p></div>")

    NEXT_EXP = re.compile('\?page=(\S+)\" class=\"pn next\">Next')
    MAX_PAGES = 0  # set to zero if pages should not be iterated

    __slots__ = ()

    @staticmethod
    def getUrlInfo(url):
        """ @param   url 
            @returns the number of bookmarks for the given url """
        return Delicious.delicious_info_retrieve(url)

    @staticmethod
    def getTagInfo(tags):
        """ @param   tags   A list of tags to retrieve information for
            @returns        the number of bookmarks using the given tags
        """
        assert(isinstance(tags, tuple) or isinstance(tags, list))
        url = Delicious._parse_tag_url(tags)
        try:
            content = Delicious._get_content(url)
            return Delicious._parse_counts(content)
        except urllib2.HTTPError:
            return 0

    @staticmethod
    def getRelatedTags(tags, retrieveTagInfo=False, pageNum=0):
        """ returns related tags for the given ones.
            @param  tags             list of tags
            @param  retrieveTagInfo  determines whether we will retrieve the tagInfo for the related tags
            @returns                 list of related tags 
        """

        assert(isinstance(tags, tuple) or isinstance(tags, list))
        tag_url = Delicious._parse_tag_url(tags)

        if not pageNum == 0:
            tag_url = '%s?page=%s' % (tag_url, pageNum)

        content = Delicious._get_content(tag_url)

        m = Delicious.NEXT_EXP.search(content)

        related_tags = (Delicious._getNGramRelatedTags(
            content) if '+' in tag_url else Delicious._getMonogramRelatedTags(content))
        related_tags_with_count = [(tag, Delicious.getTagInfo(
            (tag,)) if retrieveTagInfo else None) for tag in related_tags]

        # also find the tags in the next pages
        if m and int(pageNum) < Delicious.MAX_PAGES:
            related_tags_with_count.extend(
                Delicious.getRelatedTags(tags, pageNum=m.group(1)))

        return related_tags_with_count

    @staticmethod
    def _getMonogramRelatedTags(content):
        """ Returns the related tags for the given monogram.
            @param content of the tag's page
            @return a list of related tags

            @remark
            this method parses the delicious "Related Tags" box.
        """
        return re.findall('<span class="m" title="(\w*?)">', content, re.IGNORECASE | re.DOTALL)

    @staticmethod
    def _getNGramRelatedTags(content):
        """ Returns the related tags for the given n-gram.
            @param content of the tags' page
            @return a list of related tags

            @remark
            this method retrieves all other tags from the page and sorts
            them by the number of occurrences.
        """
        return re.findall('<span class="(?:tag-chain-item-span|tagItem)">(\w*?)</span>', content, re.IGNORECASE | re.DOTALL)

    #
    # helper functions
    #
    @staticmethod
    def _parse_counts(content):
        """ parses del.icio.us's html content and returns the number of counts """
        m = Delicious.RE_COUNT.search(content)
        if m:
            return m.group(1)
        else:
            return 0

    @staticmethod
    def _parse_tag_url(tags):
        """ parses the tag url, removes white spaces in the tags ...
            @param tuple/list of tags 
            @returns delicious tag url
        """
        tags = [quote(tag.replace(" ", "+"), safe="+") for tag in tags]
        return Delicious.DELICIOUS_TAG_URL % "+".join(tags)

    @staticmethod
    def _normalize_url(url):
        """ prepares a url for the usage by delicious"""
        if not url.endswith("/"):
            url += "/"
        return url

    @staticmethod
    def delicious_info_retrieve(url):
        assert(url.startswith("http"))

        md5_url = md5(Delicious._normalize_url(url)).hexdigest()
        request = Delicious.DELICIOUS_SERVICE_URL % md5_url
        return Delicious._parse_counts(Delicious._get_content(request))

    @staticmethod
    def _get_content(url):
        """ returns the content from delicious """
        assert(url.startswith("http"))

        f = Retrieve(Delicious.__name__).open(url)
        content = f.read()
        f.close()
        sleep(1)
        return content


if __name__ == '__main__':

    #    url = sys.argv[1].strip()
    #    print Delicious.getUrlInfo( url ), "counts"
    #    print Delicious.getTagInfo( ("debian", "linux") ), "counts"
    #    print Delicious.getRelatedTag( ("debian", "linux") ), "counts"
    print Delicious.getTagInfo(("debian", "linux")), "counts"
    print Delicious.getRelatedTags(('debian', 'linux'))



#!/usr/bin/env python

"""
@package: eWRT.input.stock
Provides access to historical stock market data

Interesting indices to support in the future:
* BALTIC DRY INDEX (BDIY:IND)
* Howe Robinson Container Index
"""

from bz2 import BZ2File
from csv import reader
from glob import glob
from os.path import dirname, basename, join as os_join
from collections import namedtuple
from datetime import datetime
from twisted.python.text import strFile

from eWRT.util.module_path import get_resource


def extract_index_name(fname): return basename(fname).split(".")[0]


DATA_DIR = get_resource(__file__, ('data', ))

Quote = namedtuple('quote', 'date last open high low change_percentage')


class StockIndex(object):
    SUPPORTED_INDICES = {extract_index_name(fname): fname
                         for fname in glob(DATA_DIR + "/*.idx.bz2")}
    SUPPORTED_FUTURES = {extract_index_name(fname): fname
                         for fname in glob(DATA_DIR + "/*.ftidx.bz2")}
    DATE_FORMAT = "%b %d, %Y"

    @classmethod
    def get_index(cls, index_name):
        """retrieves data from the given index in csv format
           @param index_name: the name of the index
        """
        return cls._read_index_filen(cls.SUPPORTED_INDICES[index_name])

    @classmethod
    def get_future(cls, future_name):
        """retrieves data from the given future in csv format
           @param future_name: the name of the future
        """
        print cls.SUPPORTED_FUTURES[future_name]
        return cls._read_index_filen(cls.SUPPORTED_FUTURES[future_name])[1:]

    @classmethod
    def _read_index_filen(cls, fname):
        """ @return: the contents of the given index file.
        """
        with BZ2File(fname) as f:
            result = [Quote._make(cls._convert_tuple(row))
                      for row in reader(f, delimiter="\t") if row]

        return result

    @classmethod
    def _convert_tuple(cls, t):
        """ converts the string in the given tuple to the respective 
            float/_date datatypes.
            @param t: the input tuple
            @return: output tuple with the correct datatypes 
        """
        #_date _last _open _high _low %_change'
        _date, _last, _open, _high, _low, _change = t
        return (datetime.strptime(_date, cls.DATE_FORMAT), float(_last),
                float(_open), float(_high), float(_low), float(_change[:-1]))


if __name__ == '__main__':
    s_list = StockIndex.get_index('dax')
    f_list = StockIndex.get_future('dax')
    delta = []
    Analysis = namedtuple('analysis', "delta volatility change_percentage")
    for s, f in zip(s_list, f_list):
        print s.date, f.date
        assert s.date == f.date
        delta.append(
            (s.date, Analysis(f.last - s.last, s.last - s.open, s.change_percentage)))

    delta.sort(key=lambda x: x[1].delta)
    print delta[:10]



#!/usr/bin/env python
"""
Usage:

> calais = Calais(submitter="MyProjectID", api_key="your-opencalais-api-key")
> text = "some text"
> calais.analyze(text)

"""

# Opencalais module
# based on python-calais v.0.2 by Jordan Dimov (jdimov@mlke.net)
# 
# (C)opyrights 2008 by Jordan Dimov <jdimov@mlke.net>
#                      Albert Weichselbraun <albert@weichselbraun.net>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import re

from random import choice
from xml.dom import minidom
from urllib import urlencode

from eWRT.access.http import Retrieve
from eWRT.config import OPENCALAIS_KEY, OPENCALAIS_CACHE_DIR, OPENCALAIS_URL, USER_AGENT
from eWRT.util.cache import DiskCache


PARAMS_XML = """
<c:params xmlns:c="http://s.opencalais.com/1/pred/" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"> 
<c:processingDirectives c:contentType="%s" c:outputFormat="Text/Simple"> 
</c:processingDirectives> 
<c:userDirectives c:allowDistribution="%s" calculateRelevanceScore="true" c:allowSearch="%s" c:externalID="%s" c:submitter="%s"> 
</c:userDirectives> 
<c:externalMetadata> 
</c:externalMetadata> 
</c:params>
"""

class Calais(object):
    submitter = USER_AGENT % "Calais"
    allow_distro = "false"
    allow_search = "false" 
    api_key = ""

    def __init__(self, submitter, api_key=OPENCALAIS_KEY, allow_distro="false", 
                 allow_search="false", cache_dir=OPENCALAIS_CACHE_DIR):
        """
        Creates a new handler for communicating with OpenCalais.  
                The parameter 'submitter' must contain a string, identifying your application.  
                'api_key' must contain a string with your OpenCalais API key (get it here: http://developer.opencalais.com/apps/register).  
        The optional parameter 'allow_distro', if set to 'true' gives OpenCalais permission to distribute the metadata extracted from your submissions.  The default value for 'allow_distro' is 'false'.  
        The optional parameter 'allow_search', if set to 'true' tells OpenCalais that future searches can be performed on the extracted metadata.  The default value for 'allow_search' is 'false'.  
        """
        assert(api_key) 
        self.submitter = submitter
        self.allow_distro = "false"
        self.allow_search = "false"
        self.api_key = api_key
        if cache_dir:
            self.cache  = DiskCache(cache_dir, cache_nesting_level=2, cache_file_suffix=".xml")

    @staticmethod
    def random_id(self):
        """
        Creates a random 10-character ID for your submission.  
        """
        chars = str.letters + str.digits
        return "".join( [ choice(chars) for i in xrange(10) ] )
    

    @staticmethod
    def content_id(text):
        """
        Creates a SHA1 hash of the text of your submission.  
        """
        try:
            import hashlib
            h = hashlib.sha1()
        except ImportError:
            import sha
            h = sha.new()

        h.update(text)
        return h.hexdigest()


    def analyze(self, text, content_type="text/txt"): 
        """ Submits 'text' to OpenCalais for analysis and memorizes the extracted metadata. 
            Set the content-type to 'text/html' if you are submitting HTML data.  
        """
        externalID = self.content_id( text )
        paramsXML = PARAMS_XML % (content_type, self.allow_distro, self.allow_search, externalID, self.submitter) 
        param = urlencode({'licenseID':self.api_key, 'content':text, 'paramsXML':paramsXML}) 
                
        # do not fetch the data again, if a file exists in the cache
        get_calais_data = lambda x: Retrieve(Calais.__name__).open(OPENCALAIS_URL, x).read()

        if self.cache is None:
            xml_data = self.unpack( get_calais_data( param ) )
        else:
            xml_data = self.unpack( self.cache.fetch( get_calais_data, param ) )

        return self.parse( xml_data )


    @staticmethod
    def unpack(calais_data):
        """ extracts calais' xml response from the data send by the calais 
            webservice 
        """
        dom = minidom.parseString(calais_data)
        return """<?xml version="1.0" encoding="utf-8"?>\n""" \
                 + dom.getElementsByTagName("string")[0].firstChild.data

    @staticmethod
    def cleanup_xml(xml_data):
        """ removes comments from xml-data-streams provided by opencalais
            @param[in] xml_data 
            @returns the xml data without any comments
        """
        result = []
        comment = False

        while '<!--' in xml_data:

            xml_data = re.sub('<!--[\s\S]*?-->', '', xml_data)
            if not re.search('<!--', xml_data):
                break
            
        return xml_data


    @staticmethod
    def parse(xml_data): 
        """ parses opencalai's xml output and returns it's dictionary representation """

        things = []

        xml_data = Calais.cleanup_xml(xml_data)

        # f= open("tmp","w"); f.write(xml_data.encode("utf8")); f.close()
        dom = minidom.parseString( xml_data.encode("utf8" ))
        
        for document in dom.getElementsByTagName("CalaisSimpleOutputFormat"):
            for annotations in document.childNodes:
                if not annotations.hasChildNodes():
                    continue
                
                if annotations.nodeName == 'Topics':
                    annotations = annotations.firstChild
                
                nodeName = annotations.nodeName
                nodeAttr = dict(annotations.attributes.items())
        
                nodeAttr.update( {'data': annotations.firstChild.data } )

                things.append( {nodeName: nodeAttr } )

        return things



"""
@package eWRT.ws
Web Service access.

"""
from __future__ import print_function


class AbstractWebSource(object):

    """a raw Web Source object"""

    NAME = None
    SUPPORTED_PARAMS = None

    # an obligatory mapping, which maps results to a standard dictionary
    # or specifies a WebSourceDocument class.
    MAPPING = None
    # MAPPING = WebSourceDocument

    def search_documents(self, search_terms, max_results=None, from_date=None,
                         to_date=None, **kwargs):
        """runs the actual search / calls the webservice / API ..."""

        raise NotImplemented

    def pre_search(self, search_params):
        """override this function to perform pre-run tasks"""

    def post_search(self, search_params):
        """override this function to perform post-run tasks"""


class AbstractIterableWebSource(AbstractWebSource):

    """web source implementing several calls to the API
    iterating over search terms and over API-specific
    maximal number of results restriction
    """

    DEFAULT_MAX_RESULTS = None  # requires only 1 api access
    DEFAULT_COMMAND = None
    DEFAULT_FORMAT = None
    RESULT_PATH = None
    DEFAULT_START_INDEX = None

    # to be locally overriden
    def search_documents(self, search_terms, max_results=DEFAULT_MAX_RESULTS,
                         from_date=None, to_date=None, **kwargs):
        """calls iterator and results' post-processor"""

        fetched = self.invoke_iterator(
            search_terms, max_results, from_date, to_date, **kwargs)

        return self.process_output(fetched, self.RESULT_PATH)

    def invoke_iterator(self, search_terms, max_results, from_date=None,
                        to_date=None, command=None, output_format=None,
                        language=None):
        """iterator: iterates over search terms and API requests"""

        for search_term in ["'{0}'".format(t) for t in search_terms]:

            if (max_results > self.DEFAULT_MAX_RESULTS):
                mid_results = self.DEFAULT_MAX_RESULTS
                # number of reguests
                for index in range(self.DEFAULT_START_INDEX,
                                   max_results + 1, self.DEFAULT_MAX_RESULTS):
                    # detect the last iteration
                    if (index + self.DEFAULT_MAX_RESULTS > max_results):
                        mid_results = max_results % self.DEFAULT_MAX_RESULTS
                    fetched = self.request(search_term, index, mid_results,
                                           from_date, to_date, command,
                                           output_format, language)
                    yield fetched

            else:
                fetched = self.request(
                    search_term, self.DEFAULT_START_INDEX, max_results,
                    from_date, to_date, command, output_format, language)
                yield fetched

    def process_output(self, results, path):
        """results' post-processor: iterates over the API responses
        and calls the output convertor
        """

        for result in results:
            for item in path(result):
                try:
                    yield self.convert_item(item)
                except Exception as e:  # ported to Python3
                    print('Error %s occured' % e)
                    continue

    def request(self, search_term, current_index, max_results,
                from_date=None, to_date=None, command=None,
                output_format=None, language=None):
        """calls the web source's API
        """

        raise NotImplemented



'''
Created on 12.06.2013

@author: streifdaniel
'''

from eWRT.ws.rest.server.service import WeblyzardService

class HelloWorld(WeblyzardService):
    
    DEFAULT_PATH = "helloworld";
    
    isLeaf = True
    
    def __init__(self, cfg=None):
        WeblyzardService.__init__(self, self.hello_world)

    def hello(self):
        return "Hello"
    
    def hello_world(self, name = "World"):
        ''' Returns "Hello [name]"
            
            @param name: the name to say hi to
            @return: greetings
        '''
        return "Hello " + name

if __name__ == '__main__':
    from twisted.web import server
    from twisted.internet import reactor
    
    root = HelloWorld()
    root.putChild(HelloWorld.DEFAULT_PATH, HelloWorld())
    server = server.Site(root)
    reactor.listenTCP(8123, server)
    reactor.run()


#!/usr/bin/env python
"""
 @package eWRT.ws.stat.coherence
 Determines how strongly two terms are connected to each other
"""

# (C)opyrights 2010 by Albert Weichselbraun <albert@weichselbraun.net>
#                      Johannes Duong <johannes.duong@wu.ac.at>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from eWRT.ws.TagInfoService import TagInfoService
from unittest import TestCase
from eWRT.util.cache import DiskCache

from math import exp


class Coherence(object):
    """ @class Coherence
        abstract class for computing the coherence between 
        terms """

    def __init__(self, dataSource, cache=True):
        """ @param[in] dataSource implementing the TagInfoService Interface """
        assert isinstance(dataSource, TagInfoService)
        self.dataSource = dataSource
        if cache == True:
            diskCache = DiskCache("./.coherence-tagcount-cache", 2)
            self.getTagCount = lambda tt: diskCache.fetchObjectId(self.dataSource.__class__.__name__ + str(tt),
                                                                  self.dataSource.getTagInfo, tt)
        else:
            self.getTagCount = self.dataSource.getTagInfo

    @staticmethod
    def getCoherence(nx, ny, nt):
        """ @param[in] nx  counts of term1
            @param[in] ny  counts of term2
            @param[in] nt counts of term1 together with term2
            @returns the coherence
        """
        raise NotImplemented

    def getTermCoherence(self, t1, t2):
        """ @param[in] t1 term1
            @param[in] t2 term2
            @returns the coherence between these two terms
        """
        nx = self.getTagCount((t1, ))
        ny = self.getTagCount((t2, ))
        nt = self.getTagCount((t1, t2))
        return self.getCoherence(nx, ny, nt)


class DiceCoherence(Coherence):
    """ @class DiceCoherence
        computes the dice coherence for the given terms """

    @staticmethod
    def getCoherence(nx, ny, nt):
        """ @param[in] nx  counts of term1
            @param[in] ny  counts of term2
            @param[in] nt counts of term1 together with term2
            @returns the coherence
        """
        try:
            return 2 * float(nt) / float(ny + ny)
        except ZeroDivisionError:
            return None


class PMICoherence(Coherence):
    """ @class PMICoherence
        computes the coherence based on the pointwise mutual
        information (PMI)
    """

    @staticmethod
    def getCoherence(nx, ny, nt):
        """ @param[in] nx  counts of term1
            @param[in] ny  counts of term2
            @param[in] nt counts of term1 together with term2
            @returns the coherence
        """
        nx, ny, nt = float(nx), float(ny), float(nt)

        nz = nx + ny + nt
        try:
            fx = (nx / nz) * exp((nx / nz) * -1)
            fy = (ny / nz) * exp((ny / nz) * -1)
            ft = (nt / nz) * exp((nt / nz) * -1)
            return ft / (fx * fy)
        except ZeroDivisionError:
            return None


class TestCoherence(TestCase):

    def testDice(self):
        """ tests the computation of the dice coefficient 
            based on the example in
              http://en.wikipedia.org/wiki/Dice's_coefficient
        """
        assert DiceCoherence.getCoherence(4, 4, 1) == 0.25

    def testPMI(self):
        """ tests the computation of the PMI based on the results
            from wilson's paper 
        """
        c = PMICoherence.getCoherence
        # one - coup whole wheat flour
        self.assertAlmostEqual(c(4710000000, 125000, 897), 0.0195069043716)
        # one coup - whole wheat flour
        self.assertAlmostEqual(c(3670000, 870000, 897), 0.00346634415814)
        # one coup whole - wheat flour
        self.assertAlmostEqual(c(4270, 2690000, 897), 0.571746307316)
        # one coup whole wheat - flour
        self.assertAlmostEqual(c(2320, 33400000.0, 897), 1.05103564089)

    def testPMIZero(self):
        """ tests the handling of PMI values of no counts are found """
        c = PMICoherence.getCoherence
        assert c(0, 12, 0) == None



#!/usr/bin/env python

""" @package eWRT.config
    evaluates ~/.eWRT/siteconfig.py and publishes the values"""

# (C)opyrights 2004-2013 by Albert Weichselbraun <albert@weichselbraun.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
from warnings import warn
from os.path import expanduser, exists, dirname, join as os_join

SYS_EWRT_CONF = "/etc/eWRT/sysconfig.py"
USR_EWRT_CONF = expanduser("~/.eWRT/siteconfig.py")

# commands used to convert various formats to text
CMD_CONV = {'html': '/usr/bin/lynx -stdin -width=20000 -force_html -nocolor -dump -nolist -nobold -pseudo_inlines=0 -assume_charset=%s -display_charset=utf8',
            'pdf': '/usr/bin/pdftotext -layout - - ',
            'doc': '/usr/bin/antiword -',
            }

# ===================================================================
# DEFAULT CONFIGURATION VALUES
# - please either create a configuration file in
#     /etc/eWRT/sysconfig or
#     ~/.eWRT/siteconfig.py
#
#   to overwrite these settings
# ===================================================================


PROXY_SERVER = ''
USER_AGENT = 'eWRT Version/0.5; MODUL <%s> +http://p.semanticlab.net/eWRT'

# default sleep time in seconds
DEFAULT_WEB_REQUEST_SLEEP_TIME = 1

# ===================================================================
# DATABASE CONFIGURATION
# ===================================================================
DATABASE_CONNECTION = {
    'db-name': {'host': 'localhost', 'dbname': 'postgres', 'username': '', 'passwd': ''},
}

# ===================================================================
# USERNAMES AND API-KEYS
# ===================================================================

# delicious
DELICIOUS_USER = ''
DELICIOUS_PASS = ''

# opencalais
OPENCALAIS_KEY = ''
OPENCALAIS_CACHE_DIR = ''
OPENCALAIS_URL = "http://api.opencalais.com/enlighten/calais.asmx/Enlighten"

# geoTagger
GEOLYZARD_URL = ''
GEOLYZARD_GAZETTEERS = ''


# amazon
AMAZON_ACCESS_KEY_DICT = {'user': 'pass',
                          }
AMAZON_ACCESS_KEY = AMAZON_ACCESS_KEY_DICT['user']

# file to copy all xml input from the amazon webservice (the content is only
# copyied if a file is specified)
AMAZON_DEBUG_FILE = ""


# ===================================================================
# PATHS AND URLs
# ===================================================================

AMAZON_LOCATIONS = {'us': 'http://webservices.amazon.com/onca/xml?Service=AWSECommerceService',
                    'uk': 'http://webservices.amazon.co.uk/onca/xml?Service=AWSECommerceService',
                    'de': 'http://webservices.amazon.de/onca/xml?Service=AWSECommerceService',
                    'jp': 'http://webservices.amazon.co.jp/onca/xml?Service=AWSECommerceService',
                    'fr': 'http://webservices.amazon.fr/onca/xml?Service=AWSECommerceService',
                    'ca': 'http://webservices.amazon.ca/onca/xml?Service=AWSECommerceService'}

# ===================================================================
# CORPUS LOCATIONS
# ===================================================================
CORPUS_DIR = '/data/corpus'
AUTOCLASS_SAMPLE = "reuters-10000-multitoken"

BBC_CORPUS = os_join(CORPUS_DIR, "news.bbc.co.uk")
BBC_CORPUS_HTML = os_join(BBC_CORPUS, "html")
BBC_CORPUS_LOW = os_join(BBC_CORPUS, "low")
BBC_CORPUS_RSS = os_join(BBC_CORPUS, "rss")
BBC_CORPUS_PRINT = os_join(BBC_CORPUS, "print")

BBC_CORPUS_TIMEFORMAT = "%a, %d %b %Y %H:%M:%S %Z"

# COMPRESSED VERSION OF THE CORPUS
BBC_CORPUS_COMPRESSED = "data/news.bbc.co.uk.tar.gz"

BING_USERNAME = ""
BING_API_KEY = ""

# facebook
FACEBOOK_API_KEY = ""
FACEBOOK_SECRET_KEY = ""
FACEBOOK_SESSION_KEY = ""
FACEBOOK_ACCESS_KEY = ""
FACEBOOK_APPLICATION_ID = "application-id"

# geonames
GEO_ENTITY_SEPARATOR = ">"

GOOGLE_USER = ""
GOOGLE_PASS = ""

# provide your google api key for browser applications (from Developers
# Console)
GOOGLE_CUSTOM_SEARCH_API_KEY = ""
# provide your Custom search engine ID
GOOGLE_CUSTOM_SEARCH_ENGINE_ID = ""

OPEN_CALAIS_KEY = ""

REUTERS_DATA_DIR = ""

SNMP_HOST_CFG = {'server': 'localhost',
                 'port': '162',
                 'community_string': 'public',
                 'oid': 'SNMPv2-SMI::enterprises.30538.5000',
                 }

SNMP_MODULE_NAME = {}
SNMP_LOG_LEVEL = {'ok': '0', 'warning': '1', 'critical': '2'}

# twitter keys .. gila
TWITTER_CONSUMER_KEY = ''
TWITTER_CONSUMER_SECRET = ''
TWITTER_ACCESS_TOKEN = ''
TWITTER_TOKEN_SECRET = ''

YAHOO_APP_ID = ''
YAHOO_SEARCH_URL = ''

YOUTUBE_API_KEY = ''

# --------------------------------------------------------------------------
#
#  Overwrite global config based on
#   a) the system configuraiton and
#   b) the user configuration.
#
# --------------------------------------------------------------------------
if exists(SYS_EWRT_CONF):
    sys.path.append(dirname(SYS_EWRT_CONF))
    try:
        from sysconfig import *
    except ImportError:
        warn("No siteconfig in present.")

if exists(USR_EWRT_CONF):
    sys.path.append(dirname(USR_EWRT_CONF))
    try:
        from siteconfig import *
    except ImportError:
        warn("Could not finde siteconfig.py in ~/.eWRT")

# $Id$



#!/usr/bin/env python

"""
::package eWRT.ws.conceptnet
Access to conceptnet data structures using its REST interface

::author: Albert Weichselbraun <albert.weichselbraun@htwchur.ch>
"""

from warnings import warn


class Node(object):
    '''
    A ConceptNet node.
    '''

    def __init__(self, url, edges):
        self.url = url
        self.edges = [e for e in edges if e['start'] == url or e['end'] == url]

    def __hash__(self):
        ''' required to support sets '''
        return self.url.__hash__()

    def __eq__(self, other):
        ''' required to support sets '''
        return self.url == other.url

    def __repr__(self):
        return "ConceptNetNode <%s>" % (self.url.encode("utf8"))

    def specificity(self):
        ''' ::return:
            a number that shows how specific this node is
            e.g. (3 for "/c/en/coal" or
                  5 for "/c/en/coal/n/energy storage)
        '''
        return len(self.url.split("/")) - 1

    @staticmethod
    def lang(node_url):
        ''' ::returns: the language of the given node_url '''
        if isinstance(node_url, dict):
            if not 'language' in node_url:
                return 'en'
            return node_url['language']
        lang = node_url.split("/")
        if len(lang) < 3:
            warn('Cannot extract language for node %s.' % (node_url, ))
            return ''
        return lang[2]



#!/usr/bin/env python

""" creates the distance database based on a wikipedia dump
"""

# -----------------------------------------------------------------------------------
# (C)opyrights 2010 by Albert Weichselbraun <albert@weichselbraun.net>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -----------------------------------------------------------------------------------

import re
import xml.parsers.expat

from sys import stdin


quote_str = lambda x: "'%s'" % x.replace("'", "''").replace("_", " ").replace("\\", "").encode("utf8").lower().replace("\"", "")

class WikiParse(object):

    RE_REDIRECT = re.compile("#REDIRECT\s*:?\s*\[\[\s*:?\s*(.*?)(?:\]\]|\||#)", re.I)
    RE_LINK     = re.compile("\[\[([^\]^\|]+)(?:\||\]\])", re.I | re.M)

    INSERT_FUNCTION = "wiki_insert"

    def __init__(self):
        self.p = xml.parsers.expat.ParserCreate()

        self.p.StartElementHandler  = self.start_element
        self.p.EndElementHandler    = self.end_element
        self.p.CharacterDataHandler = self.char_data

        self.container = None
        self._clear()
        
        self.count = 0
        self.prn   = False


    def _clear(self):
        self._wiki_concept   = ""
        self._wiki_links     = set()
        self._wiki_redirects = set()

       
    # 3 handler functions
    def start_element(self, name, attrs):
        self.container = name.lower()

    def end_element(self, name):
        if name == 'page':
            if self._wiki_concept.startswith("'Wikipedia:"):
                self.clear()
        return

        #if self.prn:
# 	    if self._wiki_concept in self._wiki_redirects:
# 		self._wiki_redirects.remove( self._wiki_concept )
# 	    if self._wiki_concept in self._wiki_links:
# 		self._wiki_links.remove( self._wiki_concept )
# 	    self._wiki_links = self._wiki_links.difference(self._wiki_redirects)  # remove redirects from links
# 
# 	    print "SELECT %s(%s, ARRAY[%s]::text[], ARRAY[%s]::text[]);" % (self.INSERT_FUNCTION,
# 		       self._wiki_concept,
# 		       ",".join( self._wiki_redirects ),
# 		       ",".join( self._wiki_links ), )
#             #else:
#             #        if self._wiki_concept == "'template:australian rules football'":
#             #        	self.prn = True
# 
#             self._clear()

    def char_data(self, data):
        if self.container == 'title':
            self._wiki_concept = quote_str(data)
            self.container = ''
        elif self.container == 'text':
            self._wiki_redirects = self._wiki_redirects.union( map(quote_str, self.RE_REDIRECT.findall(data)) )
            self._wiki_links     = self._wiki_links.union( map(quote_str, self.RE_LINK.findall(data)) )

    def parse(self, fhandle):
        self.p.ParseFile(fhandle)


if __name__ == '__main__':
    w = WikiParse()
    w.parse(stdin)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
@package: eWRT.ws.wordnet

Convenience functions to access the NLTK WordNet interface
@author: Albert Weichselbraun

Definitions:
 (a) Term - a (potentially ambiguous) n-gram representing a unit of meaning.
 (b) Synset - a concept with a well defined meaning
 (c) Lemma - a grounded list of terms representing a certain synset
'''

from itertools import chain

from nltk.corpus.reader.wordnet import Synset
from nltk.corpus import wordnet

def get_synonmys(term):
    ''' ::return: all synonyms for the given term 
                  and all its senses. '''
    return list( _get_lemmas(term) )


def get_antonyms(term):
    ''' ::return: all antonyms for the given term and 
                  all its senses.
    '''
    return [ a for a in chain(*[ lemma.antonyms() for lemma in _get_lemmas(term) ] ) if a  ]

def get_terms( lemma_list ):
    '''
    ::return: a set of strings representing the given list of lemmas
    '''
    return set( [ lemma.name for lemma in lemma_list ] )


def _get_lemmas(term):
    ''' ::return: all lemmas for the given term '''
    if isinstance(term, Synset):
        synsets = [ term ]
    else: 
        synsets = wordnet.synsets(term)

    return ( lemma for synset in synsets for lemma in synset.lemmas )




#!/usr/bin/env python

""" an example script querying book reviews from the amazon
    database """

# (C)opyrights 2008 by Albert Weichselbraun <albert@weichselbraun.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

MAX_PAGES_TO_PARSE = 10

from eWRT.ws.amazon import AmazonWS, ResultList


def getReview(asin_list):
    """ returns the reviews for the given asins """
    a = AmazonWS()

    for asin in asin_list:
        result = []
        for page in xrange(MAX_PAGES_TO_PARSE):
            xmlOutput = a.queryReview(asin, ReviewPage=str(page + 1))
            xmlParser = ResultList(
                "ItemLookupResponse/Items/Item/CustomerReviews/Review", hunt=("TotalReviewPages",))
            xmlParser.parse(xmlOutput)
            result.extend(xmlParser.xmlResult)

            print("%s: page %s/%s" % (asin, page + 1,
                                      xmlParser.huntValue.get('TotalReviewPages', 0)))
            if page + 1 >= int(xmlParser.huntValue.get('TotalReviewPages', 0)):
                break

        for review in result:
            review['Summary'] = review['Summary'].replace("'", "\\'")
            review['Content'] = review['Content'].replace("'", "\\'")
            if not "CustomerId" in review:
                print("Skipping record\n")
                continue
            print "INSERT INTO Review (asin,customerId,rating,helpfulVotes,totalVotes,date,summary,content) VALUES('%(ASIN)s', '%(CustomerId)s', %(Rating)s, %(HelpfulVotes)s, %(TotalVotes)s, '%(Date)s', '%(Summary)s', '%(Content)s');" % (review)


def exportTopBooks(number):
    """ exports the top 'number' books from amazon """

    a = AmazonWS()
    result = []
    for page in xrange(number / 10):
        xmlOutput = a.searchItem(ItemPage=str(page + 1))
        xmlParser = ResultList("ItemSearchResponse/Items/Item")
        xmlParser.parse(xmlOutput)
        result.extend(xmlParser.xmlResult)

    for dataSet in result:
        dataSet['Title'] = dataSet['Title'].replace("'", "\\'")
        if len(dataSet['ASIN']) < 10:
            continue
        assert(len(dataSet['ASIN']) == 10)
        print "INSERT INTO Book2 (asin,salesrank,title,detailPageUrl) VALUES ('%(ASIN)s', %(SalesRank)s, '%(Title)s', '%(DetailPageURL)s');" % (dataSet)


# retrieve the reviews for the given asins
asin_list = ('0385523416', '1592764053', 'B0013HA824')
getReview(asin_list)

# exports the 10 top books from amazon
exportTopBooks(10)



#!/usr/bin/env python
"""
 @package eWRT.ws.geonames.gazetteer
 fetches for an ContentID or GazEntry-ID where it is located
 e.g. for Vienna: Europe/Austria/Vienna
"""

# (C)opyrights 2009 by Heinz Lang <heinz.lang@wu.ac.at>
#                      Albert Weichselbraun <albert@weichselbraun.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from eWRT.access.db import PostgresqlDb
from eWRT.util.cache import MemoryCached
from eWRT.config import DATABASE_CONNECTION, GEO_ENTITY_SEPARATOR
from eWRT.ws.geonames.gazetteer.exception import GazetteerEntryNotFound

MIN_POPULATION = 5000


class Gazetteer(object):
    # sorting by population is a workaround for entries with multiple parents
    # (without the sorting loops occure)
    QUERY_HAS_PARENT = '''
        SELECT parent_id 
        FROM gazetteerentity ga 
        JOIN locatedin ON (ga.id = locatedin.child_id)
        JOIN gazetteerentity gb ON (gb.id = locatedin.parent_id)
        WHERE child_id = %d order by gb.population DESC LIMIT 1'''

    DEBUG = False

    def __init__(self):
        """ initializes the gazetteer object and the database connections  """
        self.db = PostgresqlDb(**DATABASE_CONNECTION['gazetteer'])
        self.db.connect()
        self.db2 = PostgresqlDb(**DATABASE_CONNECTION['geo_mapping'])
        self.db2.connect()

    def getGeoEntityDict(self, name=None, id=None, geoUrl=None):
        """ returns a list of GeoEntities matching the given information
            @param[in] name   of the Entity
            @param[in] id     the GeoNames id 
            @param[in] geoUrl the entity's dictionary
        """
        geoId = []
        if name:
            geoId.extend(self.getIdFromName(self, name))
        if id:
            geoId.append(id)
        if geoUrl:
            geoId.extend(self.getIdFromGeoUrl(geoUrl))
        return self.getGeoEntityDictFromId(geoId)

    @MemoryCached
    def getIdFromName(self, name):
        """ returns the possible GeoNames ids for the given name 
            @param[in] name
            @returns a list of GeoNames ids
        """
        query = "SELECT id FROM vw_gazetteer WHERE name='%s' ORDER BY population DESC" % (
            name.replace("'", "''"))
        res = [r['id'] for r in self.db.query(query)]
        #query = "SELECT DISTINCT entity_id FROM vw_entry_id_has_name WHERE name='%s'" % ( name.replace("'", "''") )
        return [r['id'] for r in self.db.query(query)]

    def getIdFromGeoUrl(self, geoUrl):
        """ returns the geoId for the given geoUrl 
            @param[in] geoUrl 
            @returns a list of geonames ids matching the geoUrl
        """
        geoUrl = geoUrl.split(GEO_ENTITY_SEPARATOR)

        join = []
        where = []
        for nr, name in enumerate(geoUrl):
            join.append("JOIN locatedin L%d ON (A%d.id = L%d.parent_id) JOIN gazetteerentity A%d ON (A%d.id = L%d.child_id)" % (
                nr, nr, nr, nr + 1, nr + 1, nr))
            where.append("A%d.id IN (%s)" % (nr, ", ".join(
                map(str, self.getIdFromName(self, name)))))

        query = "SELECT A%d.id AS id FROM gazetteerentity A0 %s WHERE %s;" % (
            nr, " ".join(join[:-1]), " AND ".join(where))
        return [int(r['id']) for r in self.db.query(query)]

    # @MemoryCached
    def getGeoEntityDictFromId(self, id):
        """ returns the location of the GazetteerEntry ID  
            @param id a list of geonames ids
            @return list of GeoEntities
        """
        if id:
            q = "SELECT * FROM gazetteerentity LEFT JOIN countryInfo USING(id) WHERE id IN (%s)" % ", ".join(
                map(str, id))
            res = self.db.query(q)
            if len(res) > 0:
                entities = [dict(self._getResultById(res, i).items())
                            for i in id]
                self._addGeoUrl(entities)
                return entities

            print "WARNING: no entities found for ", ", ".join(map(str, id))

        return []

    def _getResultById(self, l, id, idAttr="id"):
        """ returns the database column matching the given id 
            @param[in] l      the list of query results
            @param[in] id     the row id to return
            @param[in] idAttr attribute to consider for the id
        """
        return [e for e in l if e[idAttr] == id][0]

    def _addGeoUrl(self, entities):
        """ adds the geoUrl and level key to the given list of entities """
        for entity in entities:
            idUrl, nameUrl = self._getGeoUrl(entity['id'])
            entity['geoUrl'] = GEO_ENTITY_SEPARATOR.join(nameUrl)
            entity['idUrl'] = idUrl
            # hierarchy level of the entity (e.g. eu>at => 2)
            entity['level'] = len(nameUrl)

    def _getGeoUrl(self, id):
        """ returns the geoUrl for the given entity 
            @param[in] the geonames gazetteer id 
            @returns   two lists containing (geoIdPath, geoNamePath) 
        """
        geoNamePath = [self._getPreferredGeoName(id)]
        geoIdPath = [id]

        while id:
            parentLocationEntity = self._hasParent(id)
            if parentLocationEntity:
                parentLocationName = self._getPreferredGeoName(
                    parentLocationEntity)
                if parentLocationEntity in geoIdPath:
                    print "%s in %s" % (parentLocationName, geoNamePath)
                    break
                geoNamePath.append(parentLocationName)

            geoIdPath.append(parentLocationEntity)
            id = parentLocationEntity

        geoIdPath.reverse()
        geoNamePath.reverse()
        return (geoIdPath[1:], geoNamePath)

    # gets the preferred name for the location
    # @param id
    # @returns preferred name
    def _getPreferredGeoName(self, id):
        """ returns the preferred entry name for the given
            entity id
            @param[in] entity_id 
            @returns the geo entity's name
        """

        query = '''SELECT name FROM vw_gazetteer_tng WHERE id=%d 
                      ORDER BY 
                        (lang='en') DESC, (lang IS NULL) DESC, (lang = '') DESC,
                        (short=TRUE and short IS NOT NULL) DESC,
                        preferred DESC
                      LIMIT 1''' % (id)
        result = self.db.query(query)
        if not result:
            raise GazetteerEntryNotFound(id, query)

        return Gazetteer.DEBUG and result[0]['name'] + "(%d)" % (id) or result[0]['name']

    # checks if the given ID has a parent
    # @param ID of the child
    # @return false or ID of the parent
    def _hasParent(self, child_id):
        query = self.QUERY_HAS_PARENT % child_id
        result = self.db.query(query)

        # todo: is it necessary, that this functions can process multiple parents?
        # multiple parents (!)
        if result.__len__() > 1:
            print '### result > 1 ###'
            print '    child_id:  %s' % child_id
            print '    parent_id: %s ' % [e['parent_id'] for e in result]

        # todo: does it make sense to fetch infinite loops
        if result == []:
            return 0
        else:
            return result[0]['parent_id']



#!/usr/bin/env python

# (C)opyrights 2008-2009 by Heinz-Peter Lang <heinz@langatium.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

'''
The class NSCA-Service helps the enables to send test-results over the
NSCA service to Nagios. This is a more reliable way of sending messages
directly from programs than with SNMP. On the one hand because they are
not only associated to single service, but also as the configuration is
easier

== Configuration ==

* Install the package nsca 

 aptitude install nsca 

* On the monitoring system:

** edit the file ''/etc/nsca.cfg'': 
** set a password and an appropriate encryption method

* On the host:

** edit the file ''/etc/send_nsca.cfg''
** enter the above password and encryption method

'''
from __future__ import print_function

import os
import commands
import socket

SEND_NSCA_PATH = os.path.join(os.sep, 'usr', 'sbin', 'send_nsca')
SEND_NSCA_CONFIG = os.path.join(os.sep, 'etc', 'send_nsca.cfg')
MONITORING_SERVER = '\'tor.wu.ac.at\''


class NSCA(object):
    ''' class NSCA Service helps sending test results to Nagios '''

    @staticmethod
    # sends a NSCA message to the monitoring server
    # @param status: service status (0->OK, 1->warning, 2->critical, 3->unknown)
    # @param service_name: name of the service
    # @param src_host: name of the host the server is running on. by default the
    #                  script will run socket.gethostname()
    # @param performance:
    def send(message, status, service_name, src_host=None, performance=[],
             monitoringServer=MONITORING_SERVER):
        ''' sends the '''

        if src_host == None:

            src_host = socket.gethostname()

        STATUS = [0, 1, 2, 3]

        assert status in STATUS

        message = '{src_host};{service_name};{status};{message}'.format(
            src_host=src_host, service_name=service_name, status=status, message=message)

        if len(performance) > 0:
            message = '%s | %s ' % (message, ' '.join(
                [p.message for p in performance]))

        cmd = [SEND_NSCA_PATH, '-H', monitoringServer,
               '-d', '\';\'', '-c', SEND_NSCA_CONFIG]
        print("echo '%s' | %s " % (message, ' '.join(cmd)))
        out = commands.getstatusoutput(
            "echo '%s' | %s " % (message, ' '.join(cmd)))

        if not out[1] == '1 data packet(s) sent to host successfully.':
            print('Could not send the data packet:', out[1])
        else:
            print(out[1])


# Performance allows to add performance information to a NSCA message
class Performance(object):

    # constructor of Performance
    # @param label: name of the 'variable', e.g. time
    # @param value: value of the 'variable', e.g. 0.654
    # @param unit: unit of the value, good values are:
    #                   ... integer value
    #                 s ... seconds
    #                 % ... percentage
    #                 B, KB, MB, TB ... byte, kilo-, mega', terabyte
    #                 c ... counter
    # @param warn: threshold for warning status
    # @param critical: threshold for critical status
    # @param min: minimum value
    # @param max: maximum value
    def __init__(self, label, value, unit='', warn='', critical='', min='', max=''):

        label = '\'%s\'' % label
        assert isinstance(value, (int, long, float, complex))
        assert unit in ('', 's', '%', 'B', 'KB', 'MB', 'TB', 'c')

        self.message = '%s=%s%s;%s;%s;%s;%s' % (
            label, value, unit, warn, critical, min, max)
        print(self.message)


if __name__ == '__main__':

    NSCA.send('just a test!', 0, 'webLyzard notifications',
              'tor.wu.ac.at', [Performance('test', 3, 's')])



#!/usr/bin/env python

'''
Created on Aug 30, 2014

:author: svakulenko
'''

from eWRT.ws.rest import RESTClient
from eWRT.ws import AbstractIterableWebSource


def RESULT_PATH(x): return x['items']  # path to the results


class CustomSearch(AbstractIterableWebSource):

    """wrapper for the Google Custom Search API"""

    NAME = "Google Custom Search"
    ROOT_URL = 'https://www.googleapis.com/customsearch'
    DEFAULT_MAX_RESULTS = 10  # requires only 1 api access
    SUPPORTED_PARAMS = ['command', 'output_format', 'language']
    DEFAULT_COMMAND = 'v1'
    DEFAULT_FORMAT = 'json'
    DEFAULT_START_INDEX = 1

    DEFAULT_RESULT_LANGUAGE = 'lang_de'  # lang_en

    MAPPING = {'date': ('valid_from', 'convert_date'),
               'text': ('content', None),
               'title': 'Title',
               }

    def __init__(self, api_key, engine_id, api_url=ROOT_URL):
        """fixes the credentials and initiates the RESTClient"""

        assert(api_key)
        self.api_key = api_key

        assert(engine_id)
        self.engine_id = engine_id

        self.api_url = api_url
        self.client = RESTClient(self.api_url, authentification_method='basic')

    def search_documents(self, search_terms, max_results=DEFAULT_MAX_RESULTS,
                         from_date=None, to_date=None, command=DEFAULT_COMMAND,
                         output_format=DEFAULT_FORMAT,
                         language=DEFAULT_RESULT_LANGUAGE):
        """calls iterator and results' post-processor"""

        fetched = self.invoke_iterator(search_terms, max_results, from_date,
                                       to_date, command, output_format, language)

        return self.process_output(fetched, RESULT_PATH)

    def request(self, search_term, current_index,
                max_results=DEFAULT_MAX_RESULTS, from_date=None, to_date=None,
                command=DEFAULT_COMMAND, output_format=DEFAULT_FORMAT,
                language=DEFAULT_RESULT_LANGUAGE):
        """calls Google Custom Search API"""
        parameters = {'q': '"%s"' % search_term,
                      'alt': output_format,
                      'cx': self.engine_id,
                      'key': self.api_key,
                      'num': max_results,
                      'start': current_index,
                      'sort': 'date'}

        # set language
        if language is not None and not language.startswith('lang_') and len(language) == 2:
            language = 'lang_{}'.format(language)
        if language is not None and len(language) == 7:
            parameters['lr'] = language

        response = self.client.execute(command, query_parameters=parameters)
        return response

    @classmethod
    def convert_item(cls, item):
        """output convertor: applies a mapping to convert
        the result to the required format
        """

        result = {'url': item['link'],
                  'title': item['title'],
                  }

        return result



#!/usr/bin/env python
"""
   @package eWRT.ws.wikipedia.distance
   wikipedia "click" distance between concepts
"""
 
# -----------------------------------------------------------------------------------
# (C)opyrights 2010 by Albert Weichselbraun <albert@weichselbraun.net>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -----------------------------------------------------------------------------------
 
__author__   = "albert"
__revision__ = "$Revision: 1 $"
 
from eWRT.access.db import PostgresqlDb
from eWRT.config import DATABASE_CONNECTION
 
class WikiDistance(object):
     
    escape_terms = staticmethod( lambda tt:", ".join( ["'%s'" % t.replace("'", "''") for t in tt]) ) 
     
    def __init__(self):
        """ open the database connection """
        dbParam = DATABASE_CONNECTION['wikipedia'].copy()
        dbParam['connect'] = False
        self.db = PostgresqlDb( **dbParam)
 
    def isSibling(self, t1, t2):
        """ determines whether the given concepts are siblings
            @param[in] t1   first term
            @param[in] t2   second term
            @return True if both terms are siblings
        """
        tt = WikiDistance.escape_terms( (t1,t2) ) 
        with self.db as c:
            q="SELECT * FROM vw_siblings WHERE dname IN (%s) AND sname IN (%s)" % (tt, tt)
            cnt = len(c.query(q))
        return cnt > 0 
 
    def isSameAs(self, t1, t2):
        """ determines whether the given terms refer to the same concept
            @param[in] t1   first term
            @param[in] t2   second term
            @return True if both terms refer to the same concept
        """
        tt = WikiDistance.escape_terms( (t1,t2) )
        with self.db as c:
            q="SELECT * FROM vw_sameas WHERE dname IN (%s) AND sname IN (%s)" % (tt, tt)
            cnt = len(c.query(q))
        return cnt > 0 
 
 
def p_isSibling( concepts ):
    """ 
        @param[in] concepts a tuple containing the two concepts to check
 
        @remarks
        helper function for the testMultiProcessing test
    """
    assert len(concepts) == 2
    w = WikiDistance()
    return w.isSameAs( *concepts ), w.isSibling( *concepts )




#!/usr/bin/env python

''' @package eWRT.util.cache
    caches arbitrary objects
'''
from __future__ import print_function

import redis
import pickle

from gzip import GzipFile
from hashlib import sha1
from operator import itemgetter
from os import makedirs, remove, getpid, link
from os.path import join, exists, dirname, basename, join
from socket import gethostname
from time import time

from eWRT.util.pickleIterator import WritePickleIterator, ReadPickleIterator


# (C)opyrights 2008-2015 by Albert Weichselbraun <albert@weichselbraun.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
__author__ = "Albert Weichselbraun"
__copyright__ = "GPL"


try:
    from cPickle import dump, load
except ImportError:
    from pickle import dump, load


def get_unique_temp_file(fname): return join(dirname(fname),
                                             "_%s-%s-%d" % (basename(fname),
                                                            gethostname(), getpid()))


class Cache(object):
    ''' An abstract class for caching functions '''

    def __init__(self, fn=None):
        self.fn = fn

    def __call__(self, *args, **kargs):
        ''' retrieves the result using self.fn as function and
            the cache.
            ::param args:     arguments
            ::param kargs:    optional keyword arguments
        '''
        assert self.fn
        return self.fetch(self.fn, *args, **kargs)

    def __delitem__(self, key):
        ''' Removes items from the cache
        ::param key: the item to remove
        '''
        raise NotImplementedError

    def fetchObjectId(self, key, fetch_function, *args, **kargs):
        ''' Fetches a object from the cache or computes it by calling the
            fetch_function.
            The key helps to determine whether the object is already in
            the cache or not.
        '''
        raise NotImplementedError

    def fetch(self, fetch_function, *args, **kargs):
        ''' Fetches a object from the cache or computes it by calling the
            fetch_function.
            The objectId is computed based on the function arguments
        '''
        raise NotImplementedError

    @staticmethod
    def getKey(*args, **kargs):
        ''' returns the key for a set of function parameters '''
        return (args, tuple(kargs.items()))

    @staticmethod
    def getObjectId(obj):
        ''' returns an identifier representing the object '''
        return sha1(repr(obj).encode("utf8")).hexdigest()


class DiskCache(Cache):
    ''' @class DiskCache
        Caches abitrary functions based on the function's arguments (fetch) or
        on a user defined key (fetchObjectId)

        @remarks
        This version of DiskCached is threadsafe
    '''

    def __init__(self, cache_dir, cache_nesting_level=0, cache_file_suffix="", fn=None):
        ''' initializes the Cache object
            ::param cache_dir: the cache base directory
            ::param cache_nesting_level: optional number of nesting level (0)
            ::param cache_file_suffix: optional suffix for cache files
            ::param fn: function to cache (optional; required for directly calling the class
                          using __call__
        '''
        Cache.__init__(self, fn)
        self.cache_dir = cache_dir
        self.cache_file_suffix = cache_file_suffix
        self.cache_nesting_level = cache_nesting_level

        self._cache_hit = 0
        self._cache_miss = 0

    def fetch(self, fetch_function, *args, **kargs):
        ''' fetches the object with the given id, querying
             a) the cache and
             b) the fetch_function
            if the fetch_function is called, the functions result is saved
            in the cache

            ::param fetch_function: function to call if the result is not in the cache
            ::param args:   arguments
            ::param kargs:  optional keyword arguments

            ::returns: the object (retrieved from the cache or computed)
        '''
        objectId = self.getKey(*args, **kargs)
        return self.fetchObjectId(objectId, fetch_function, *args, **kargs)

    def __contains__(self, key):
        ''' returns whether the key is already stored in the cache '''
        cache_file = self._get_fname(self.getObjectId(key))
        return exists(cache_file)

    def __delitem__(self, key):
        ''' removes the given item from the cache '''
        cache_file = self._get_fname(self.getObjectId(key))
        remove(cache_file)

    def fetchObjectId(self, key, fetch_function, *args, **kargs):
        ''' fetches the object with the given id, querying
             * the cache and
             * the fetch_function
            if the fetch_function is called, the functions result is saved
            in the cache

            ::param key:      key to fetch
            ::param fetch_function: function to call if the result is not in the cache
            ::param args:     arguments
            ::param kargs:    optional keyword arguments

            ::returns: the object (retrieved from the cache or computed)
        '''
        cache_file = self._get_fname(self.getObjectId(key))
        if exists(cache_file):
            #
            # case 1: cache hit - return the cached result
            #
            self._cache_hit += 1
            with GzipFile(cache_file) as f:
                return load(f)

        #
        # case 2: cache miss
        # - compute and cache the result
        #
        temp_file = get_unique_temp_file(cache_file)

        self._cache_miss += 1
        obj = fetch_function(*args, **kargs)

        # Do not cache None
        if obj == None:
            return obj

        with GzipFile(temp_file, "w") as f:
            dump(obj, f)

        try:
            link(temp_file, cache_file)
        # ignore file exists errors
        except OSError as e:
            if e.errno != 17:
                raise e

        remove(temp_file)
        return obj

    def _remove(self, fname):
        ''' removes the given files (if it exists) '''
        try:
            remove(fname)
        except OSError:
            pass

    def getCacheStatistics(self):
        ''' returns statistics regarding the cache's hit/miss ratio '''
        return {'cache_hits': self._cache_hit, 'cache_misses': self._cache_miss}

    def _get_fname(self, obj_id):
        ''' Computes the filename of the file with the given
            object identifier and creates the required directory
            structure (if necessary).

            @returns the full path of the given object's cache file
        '''
        assert len(obj_id) >= self.cache_nesting_level

        obj_dir = join(
            *([self.cache_dir] + list(obj_id[:self.cache_nesting_level])))
        if not exists(obj_dir):
            try:
                makedirs(obj_dir)
            except OSError:            # required for multithreading
                pass

        return join(obj_dir, obj_id + self.cache_file_suffix)


class DiskCached(object):
    ''' Decorator based on Cache for caching arbitrary function calls
        usage:
          @DiskCached("./cache/myfunction")
          def myfunction(*args):

        @remarks
        This version of DiskCached is threadsafe
    '''
    __slots__ = ('cache', )

    def __init__(self, cache_dir, cache_nesting_level=0, cache_file_suffix=""):
        ''' initializes the Cache object
            ::param fn:                  the function to cache
            ::param cache_dir:           the cache base directory
            ::param cache_nesting_level: optional number of nesting level (0)
            ::param cache_file_suffix:   optional suffix for cache files
        '''
        self.cache = DiskCache(
            cache_dir, cache_nesting_level, cache_file_suffix)

    def __call__(self, fn):
        self.cache.fn = fn
        return self.cache


class MemoryCache(Cache):
    '''
        @class MemoryCached

        Caches abitrary functions based on the function's arguments (fetch) or
        on a user defined key (fetchObjectId)
    '''
    __slots__ = ('max_cache_size', '_cacheData', '_usage')

    def __init__(self, max_cache_size=0, fn=None):
        ''' initializes the Cache object '''
        Cache.__init__(self, fn)
        self._cacheData = {}
        self._usage = {}
        self.max_cache_size = max_cache_size

    def fetch(self, fetch_function, *args, **kargs):
        key = self.getKey(*args, **kargs)
        return self.fetchObjectId(key, fetch_function, *args, **kargs)

    def fetchObjectId(self, key, fetch_function, *args, **kargs):
        # update the object's last usage time stamp
        key = self.getObjectId(key)
        self._usage[key] = time()
        try:
            return self._cacheData[key]
        except KeyError:
            obj = fetch_function(*args, **kargs)
            if obj != None:
                self.garbage_collect_cache()
                self._cacheData[key] = obj
            return obj

    def __contains__(self, key):
        ''' returns whether the key is already stored in the cache '''
        return self.getObjectId(key) in self._cacheData

    def __delitem__(self, key):
        ''' removes the given item from the cache '''
        del self._cacheData[self.getObjectId(key)]

    def garbage_collect_cache(self):
        ''' removes the object which have not been in use for the
            longest time '''
        if self.max_cache_size == 0 or len(self._cacheData) <= self.max_cache_size:
            return

        (key, _) = sorted(self._usage.items(),
                          key=itemgetter(1), reverse=True).pop()
        del self._usage[key]
        del self._cacheData[key]


class MemoryCached(MemoryCache):
    ''' Decorator based on MemoryCache for caching arbitrary function calls
        usage:
          @MemoryCached or @MemoryCached(max_cache_size)
          def myfunction(*args):            ...
    '''

    def __init__(self, arg):
        ''' initializes the MemoryCache object
            ::param arg: either the max_cache_size or the function to call
        '''
        if hasattr(arg, '__call__'):
            MemoryCache.__init__(self)
            self._fn = arg
        else:
            MemoryCache.__init__(self, max_cache_size=arg)
            self._fn = None

    def __call__(self, *args, **kargs):
        if self._fn == None:
            fn = args[0]

            def wrapped_fn(*args, **kargs):
                return self.fetch(fn, *args, **kargs)
            return wrapped_fn
        else:
            return self.fetch(self._fn, *args, **kargs)


class IterableCache(DiskCache):
    ''' caches arbitrary iterable content identified by an identifier '''

    def __iter__(self):
        return self

    def fetchObjectId(self, key, function, *args, **kargs):
        ''' fetches the object with the given id, querying
             a) the cache and
             b) the function
            if the function is called, the functions result is saved
            in the cache

            ::param key:      key to fetch
            ::param function: function to call if the result is not in the cache
            ::param args:     arguments
            ::param kargs:    optional keyword arguments

            ::returns: the object (retrieved from the cache or computed)
        '''
        cache_file = self._get_fname(self.getObjectId(key))

        if exists(cache_file):
            self._cached = True
            self._pickle_iterator = ReadPickleIterator(cache_file)
        else:
            self._fetch_function_iterator = function(*args, **kargs).__iter__()
            self._cached = False
            self._pickle_iterator = WritePickleIterator(cache_file)

        return self

    def next(self):
        return self._read_next_element() if self._cached else self._cache_next_element()

    def _cache_next_element(self):
        ''' a) retrieves the next element from the fetch function
            b) writes the data to the cache
            c) passes the data through to the calling element
        '''
        self._cache_miss += 1
        try:
            obj = next(self._fetch_function_iterator)
            self._pickle_iterator.dump(obj)
            return obj
        except StopIteration:
            self._pickle_iterator.close()
            raise StopIteration

    def _read_next_element(self):
        ''' returns the next element from the cache '''
        self._cache_hit += 1
        try:
            return next(self._pickle_iterator)
        except IOError:
            self._pickle_iterator.close()
            raise StopIteration


class RedisCache(Cache):

    def __init__(self, max_cache_size=0, fn=None, host='localhost', port=6379, db=0):
        ''' initializes the Cache object '''
        Cache.__init__(self, fn)
        self._cacheData = redis.StrictRedis(host=host, port=port, db=db)
        self._usage = redis.StrictRedis(host=host, port=port, db=1)
        try:
            self._cacheData.ping()
            self.max_cache_size = max_cache_size
        except:
            print("RedisCache requires a running Redis server.")

    def fetch(self, fetch_function, *args, **kargs):
        key = self.getKey(*args, **kargs)
        return self.fetchObjectId(key, fetch_function, *args, **kargs)

    def fetchObjectId(self, key, fetch_function, *args, **kargs):
        # update the object's last usage time stamp
        key = self.getObjectId(key)
        self._usage[key] = time()
        # pickling is necessary because Redis turns every input into
        # a string
        try:
            return(pickle.loads(self._cacheData[key]))
        except KeyError:
            obj = fetch_function(*args, **kargs)
            if obj != None:
                self.garbage_collect_cache()
                p_obj = pickle.dumps(obj)
                self._cacheData[key] = p_obj
            return(obj)

    def garbage_collect_cache(self):
        ''' removes the object which have not been in use for the
            longest time '''
        if self.max_cache_size == 0 or self._cacheData.dbsize() < self.max_cache_size:
            return
        usage_dict = {k: self._usage[k] for k in self._usage.keys()}
        (key, _) = sorted(usage_dict.items(),
                          key=itemgetter(1), reverse=True).pop()
        del self._usage[key]
        del self._cacheData[key]


class RedisCached(RedisCache):
    ''' Decorator based on MemoryCache for caching arbitrary function calls
        usage:
          @MemoryCached or @MemoryCached(max_cache_size)
          def myfunction(*args):            ...
    '''

    def __init__(self, arg):
        ''' initializes the RedisCache object
            ::param arg: either the max_cache_size or the function to call
        '''
        if hasattr(arg, '__call__'):
            RedisCache.__init__(self)
            self._fn = arg
        else:
            RedisCache.__init__(self, **arg)
            self._fn = None

    def __call__(self, *args, **kargs):
        if self._fn == None:
            fn = args[0]

            def wrapped_fn(*args, **kargs):
                return self.fetch(fn, *args, **kargs)
            return wrapped_fn
        else:
            return self.fetch(self._fn, *args, **kargs)



'''
Created on 29.05.2013

This modules help increasing the expiry time of the access token and returns 
a new one. 

:author: heinz-peterlang
'''
import urlparse

from eWRT.access.http import Retrieve
from eWRT.config import (FACEBOOK_APPLICATION_ID, FACEBOOK_SECRET_KEY,
                         FACEBOOK_ACCESS_KEY)

API_URL = 'https://graph.facebook.com/oauth/access_token?client_id={client_id}&client_secret={client_secret}&grant_type=fb_exchange_token&fb_exchange_token={access_token}'


def get_new_access_token(client_id=FACEBOOK_APPLICATION_ID,
                         client_secret=FACEBOOK_SECRET_KEY,
                         access_token=FACEBOOK_ACCESS_KEY):
    ''' '''
    url = API_URL.format(client_id=client_id,
                         client_secret=client_secret,
                         access_token=access_token)

    retrieve = Retrieve('fb')
    x = retrieve.open(url)
    result = x.read()
    new_access_token = access_token

    for key, param in urlparse.parse_qs(result).iteritems():
        print key, param
        if key == 'access_token':
            if isinstance(param, list):
                param = param[0]

            if param == access_token:
                print 'access token still the same'
            else:
                print 'got new access_token %s' % param
                new_access_token = param

    return new_access_token


if __name__ == '__main__':
    get_new_access_token()



# ----------------------------------------------------------
# AdvaS Advanced Search
# module for phonetic algorithms
#
# (C) 2002 - 2005 Frank Hofmann, Chemnitz, Germany
# email fh@efho.de
# ----------------------------------------------------------

# changed 2005-01-24

import string
import re

def soundex (term):
    "Return the soundex value to a string argument."

    # Create and compare soundex codes of English words.
    #
    # Soundex is an algorithm that hashes English strings into
    # alpha-numerical value that represents what the word sounds
    # like. For more information on soundex and some notes on the
    # differences in implemenations visit:
    # http://www.bluepoof.com/Soundex/info.html
    #
    # This version modified by Nathan Heagy at Front Logic Inc., to be
    # compatible with php's soundexing and much faster.
    #
    # eAndroid / Nathan Heagy / Jul 29 2000
    # changes by Frank Hofmann / Jan 02 2005

    # generate translation table only once. used to translate into soundex numbers
    #table = string.maketrans('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ', '0123012002245501262301020201230120022455012623010202')
    table = string.maketrans('ABCDEFGHIJKLMNOPQRSTUVWXYZ', '01230120022455012623010202')

    # check parameter
    if not term:
        return "0000" # could be Z000 for compatibility with other implementations
    # end if

        # convert into uppercase letters
    term = string.upper(term)
    first_char = term[0]

    # translate the string into soundex code according to the table above
    term = string.translate(term[1:], table)

    # remove all 0s
    term = term.replace("0", "")

    # remove duplicate numbers in-a-row
    str2 = first_char
    for x in term:
        if x != str2[-1]:
            str2 = str2 + x
        # end if
    # end for

    # pad with zeros
    str2 = str2+"0"*len(str2)

    # take the first four letters
    return_value = str2[:4]

    # return value
    return return_value

def metaphone (term):
    "returns metaphone code for a given string"

    # implementation of the original algorithm from Lawrence Philips
    # extended/rewritten by M. Kuhn
    # improvements with thanks to John Machin <sjmachin@lexicon.net>

    # define return value
    code = ""

    i = 0
    term_length = len(term)

    if (term_length == 0):
        # empty string ?
        return code
    # end if

    # extension #1 (added 2005-01-28)
    # convert to lowercase
    term = term.lower()

    # extension #2 (added 2005-01-28)
    # remove all non-english characters, first
    term = re.sub(r'[^a-z]', '', term)
    if len(term) == 0:
        # nothing left
        return code
    # end if

    # extension #3 (added 2005-01-24)
    # conflate repeated letters
    firstChar = term[0]
    str2 = firstChar
    for x in term:
        if x != str2[-1]:
            str2 = str2 + x
        # end if
    # end for

    # extension #4 (added 2005-01-24)
    # remove any vowels unless a vowel is the first letter
    firstChar = str2[0]
    str3 = firstChar
    for x in str2[1:]:
        if (re.search(r'[^aeiou]', x)):
            str3 = str3 + x
        # end if
    # end for

    term = str3
    term_length = len(term)
    if term_length == 0:
        # nothing left
        return code
    # end if

    # check for exceptions
    if (term_length > 1):
        # get first two characters
        first_chars = term[0:2]

        # build translation table
        table = {
            "ae":"e",
            "gn":"n",
            "kn":"n",
            "pn":"n",
            "wr":"n",
            "wh":"w"
        }

        if first_chars in table.keys():
            term = term[2:]
            code = table[first_chars]
            term_length = len(term)
        # end if

    elif (term[0] == "x"):
        term = ""
        code = "s"
        term_length = 0
    # end if

    # define standard translation table
    st_trans = {
        "b":"b",
        "c":"k",
        "d":"t",
        "g":"k",
        "h":"h",
        "k":"k",
        "p":"p",
        "q":"k",
        "s":"s",
        "t":"t",
        "v":"f",
        "w":"w",
        "x":"ks",
        "y":"y",
        "z":"s"
    }

    i = 0
    while (i<term_length):
        # init character to add, init basic patterns
        add_char = ""
        part_n_2 = ""
        part_n_3 = ""
        part_n_4 = ""
        part_c_2 = ""
        part_c_3 = ""

        # extract a number of patterns, if possible
        if (i < (term_length - 1)):
            part_n_2 = term[i:i+2]

            if (i>0):
                part_c_2 = term[i-1:i+1]
                part_c_3 = term[i-1:i+2]
            # end if
        # end if

        if (i < (term_length - 2)):
            part_n_3 = term[i:i+3]
        # end if

        if (i < (term_length - 3)):
            part_n_4 = term[i:i+4]
        # end if

        # use table with conditions for translations
        if (term[i] == "b"):
            add_char = st_trans["b"]
            if (i == (term_length - 1)):
                if (i>0):
                    if (term[i-1] == "m"):
                        add_char = ""
                    # end if
                # end if
            # end if
        elif (term[i] == "c"):
            add_char = st_trans["c"]
            if (part_n_2 == "ch"):
                add_char = "x"
            elif (re.search(r'c[iey]', part_n_2)):
                add_char = "s"
            # end if

            if (part_n_3 == "cia"):
                add_char = "x"
            # end if

            if (re.search(r'sc[iey]', part_c_3)):
                add_char = ""
            # end if

        elif (term[i] == "d"):
            add_char = st_trans["d"]
            if (re.search(r'dg[eyi]', part_n_3)):
                add_char = "j"
            # end if

        elif (term[i] == "g"):
            add_char = st_trans["g"]

            if (part_n_2 == "gh"):
                if (i == (term_length - 2)):
                    add_char = ""
                # end if
            elif (re.search(r'gh[aeiouy]', part_n_3)):
                add_char = ""
            elif (part_n_2 == "gn"):
                add_char = ""
            elif (part_n_4 == "gned"):
                add_char = ""
            elif (re.search(r'dg[eyi]',part_c_3)):
                add_char = ""
            elif (part_n_2 == "gi"):
                if (part_c_3 != "ggi"):
                    add_char = "j"
                # end if
            elif (part_n_2 == "ge"):
                if (part_c_3 != "gge"):
                    add_char = "j"
                # end if
            elif (part_n_2 == "gy"):
                if (part_c_3 != "ggy"):
                    add_char = "j"
                # end if
            elif (part_n_2 == "gg"):
                add_char = ""
            # end if
        elif (term[i] == "h"):
            add_char = st_trans["h"]
            if (re.search(r'[aeiouy]h[^aeiouy]', part_c_3)):
                add_char = ""
            elif (re.search(r'[csptg]h', part_c_2)):
                add_char = ""
            # end if
        elif (term[i] == "k"):
            add_char = st_trans["k"]
            if (part_c_2 == "ck"):
                add_char = ""
            # end if
        elif (term[i] == "p"):
            add_char = st_trans["p"]
            if (part_n_2 == "ph"):
                add_char = "f"
            # end if
        elif (term[i] == "q"):
            add_char = st_trans["q"]
        elif (term[i] == "s"):
            add_char = st_trans["s"]
            if (part_n_2 == "sh"):
                add_char = "x"
            # end if

            if (re.search(r'si[ao]', part_n_3)):
                add_char = "x"
            # end if
        elif (term[i] == "t"):
            add_char = st_trans["t"]
            if (part_n_2 == "th"):
                add_char = "0"
            # end if

            if (re.search(r'ti[ao]', part_n_3)):
                add_char = "x"
            # end if
        elif (term[i] == "v"):
            add_char = st_trans["v"]
        elif (term[i] == "w"):
            add_char = st_trans["w"]
            if (re.search(r'w[^aeiouy]', part_n_2)):
                add_char = ""
            # end if
        elif (term[i] == "x"):
            add_char = st_trans["x"]
        elif (term[i] == "y"):
            add_char = st_trans["y"]
        elif (term[i] == "z"):
            add_char = st_trans["z"]
        else:
            # alternative
            add_char = term[i]
        # end if

        code = code + add_char
        i += 1
    # end while

    # return metaphone code
    return code

def nysiis (term):
    "returns New York State Identification and Intelligence Algorithm (NYSIIS) code for the given term"

    code = ""

    i = 0
    term_length = len(term)

    if (term_length == 0):
        # empty string ?
        return code
    # end if

    # build translation table for the first characters
    table = {
        "mac":"mcc",
        "ph":"ff",
        "kn":"nn",
        "pf":"ff",
        "k":"c",
        "sch":"sss"
    }

    for table_entry in table.keys():
        table_value = table[table_entry]    # get table value
        table_value_len = len(table_value)    # calculate its length
        first_chars = term[0:table_value_len]
        if (first_chars == table_entry):
            term = table_value + term[table_value_len:]
            break
        # end if
    # end for

    # build translation table for the last characters
    table = {
        "ee":"y",
        "ie":"y",
        "dt":"d",
        "rt":"d",
        "rd":"d",
        "nt":"d",
        "nd":"d",
    }

    for table_entry in table.keys():
        table_value = table[table_entry]    # get table value
        table_entry_len = len(table_entry)    # calculate its length
        last_chars = term[(0 - table_entry_len):]
        #print last_chars, ", ", table_entry, ", ", table_value
        if (last_chars == table_entry):
            term = term[:(0 - table_value_len + 1)] + table_value
            break
        # end if
    # end for

    # initialize code
    code = term

    # transform ev->af
    code = re.sub(r'ev', r'af', code)

    # transform a,e,i,o,u->a
    code = re.sub(r'[aeiouy]', r'a', code)

    # transform q->g
    code = re.sub(r'q', r'g', code)

    # transform z->s
    code = re.sub(r'z', r's', code)

    # transform m->n
    code = re.sub(r'm', r'n', code)

    # transform kn->n
    code = re.sub(r'kn', r'n', code)

    # transform k->c
    code = re.sub(r'k', r'c', code)

    # transform sch->sss
    code = re.sub(r'sch', r'sss', code)

    # transform ph->ff
    code = re.sub(r'ph', r'ff', code)

    # transform h-> if previous or next is nonvowel -> previous
    occur = re.findall(r'([a-z]{0,1}?)h([a-z]{0,1}?)', code)
    #print occur
    for occur_group in occur:
        occur_item_previous = occur_group[0]
        occur_item_next = occur_group[1]

        if ((re.match(r'[^aeiouy]', occur_item_previous)) or (re.match(r'[^aeiouy]', occur_item_next))):
            if (occur_item_previous != ""):
                # make substitution
                code = re.sub (occur_item_previous + "h", occur_item_previous * 2, code, 1)
            # end if
        # end if
    # end for

    # transform w-> if previous is vowel -> previous
    occur = re.findall(r'([aeiouy]{1}?)w', code)
    #print occur
    for occur_group in occur:
        occur_item_previous = occur_group[0]
        # make substitution
        code = re.sub (occur_item_previous + "w", occur_item_previous * 2, code, 1)
    # end for

    # check last character
    # -s, remove
    code = re.sub (r's$', r'', code)
    # -ay, replace by -y
    code = re.sub (r'ay$', r'y', code)
    # -a, remove
    code = re.sub (r'a$', r'', code)

    # return nysiis code
    return code

def caverphone (term):
    "returns the language key using the caverphone algorithm 2.0"

    # Developed at the University of Otago, New Zealand.
    # Project: Caversham Project (http://caversham.otago.ac.nz)
    # Developer: David Hood, University of Otago, New Zealand
    # Contact: caversham@otago.ac.nz
    # Project Technical Paper: http://caversham.otago.ac.nz/files/working/ctp150804.pdf
    # Version 2.0 (2004-08-15)

    code = ""

    i = 0
    term_length = len(term)

    if (term_length == 0):
        # empty string ?
        return code
    # end if

    # convert to lowercase
    code = term.lower()

    # remove anything not in the standard alphabet (a-z)
    code = re.sub(r'[^a-z]', '', code)

    # remove final e
    if code.endswith("e"):
        code = code[:-1]

    # if the name starts with cough, rough, tough, enough or trough -> cou2f (rou2f, tou2f, enou2f, trough)
    code = re.sub(r'^([crt]|(en)|(tr))ough', r'\1ou2f', code)

    # if the name starts with gn -> 2n
    code = re.sub(r'^gn', r'2n', code)

    # if the name ends with mb -> m2
    code = re.sub(r'mb$', r'm2', code)

    # replace cq -> 2q
    code = re.sub(r'cq', r'2q', code)

    # replace c[i,e,y] -> s[i,e,y]
    code = re.sub(r'c([iey])', r's\1', code)

    # replace tch -> 2ch
    code = re.sub(r'tch', r'2ch', code)

    # replace c,q,x -> k
    code = re.sub(r'[cqx]', r'k', code)

    # replace v -> f
    code = re.sub(r'v', r'f', code)

    # replace dg -> 2g
    code = re.sub(r'dg', r'2g', code)

    # replace ti[o,a] -> si[o,a]
    code = re.sub(r'ti([oa])', r'si\1', code)

    # replace d -> t
    code = re.sub(r'd', r't', code)

    # replace ph -> fh
    code = re.sub(r'ph', r'fh', code)

    # replace b -> p
    code = re.sub(r'b', r'p', code)

    # replace sh -> s2
    code = re.sub(r'sh', r's2', code)

    # replace z -> s
    code = re.sub(r'z', r's', code)

    # replace initial vowel [aeiou] -> A
    code = re.sub(r'^[aeiou]', r'A', code)

    # replace all other vowels [aeiou] -> 3
    code = re.sub(r'[aeiou]', r'3', code)

    # replace j -> y
    code = re.sub(r'j', r'y', code)

    # replace an initial y3 -> Y3
    code = re.sub(r'^y3', r'Y3', code)

    # replace an initial y -> A
    code = re.sub(r'^y', r'A', code)

    # replace y -> 3
    code = re.sub(r'y', r'3', code)

    # replace 3gh3 -> 3kh3
    code = re.sub(r'3gh3', r'3kh3', code)

    # replace gh -> 22
    code = re.sub(r'gh', r'22', code)

    # replace g -> k
    code = re.sub(r'g', r'k', code)

    # replace groups of s,t,p,k,f,m,n by its single, upper-case equivalent
    for single_letter in ["s", "t", "p", "k", "f", "m", "n"]:
        otherParts = re.split(single_letter + "+", code)
        code = string.join(otherParts, string.upper(single_letter))

    # replace w[3,h3] by W[3,h3]
    code = re.sub(r'w(h?3)', r'W\1', code)

    # replace final w with 3
    code = re.sub(r'w$', r'3', code)

    # replace w -> 2
    code = re.sub(r'w', r'2', code)

    # replace h at the beginning with an A
    code = re.sub(r'^h', r'A', code)

    # replace all other occurrences of h with a 2
    code = re.sub(r'h', r'2', code)

    # replace r3 with R3
    code = re.sub(r'r3', r'R3', code)

    # replace final r -> 3
    code = re.sub(r'r$', r'3', code)

    # replace r with 2
    code = re.sub(r'r', r'2', code)

    # replace l3 with L3
    code = re.sub(r'l3', r'L3', code)

    # replace final l -> 3
    code = re.sub(r'l$', r'3', code)

    # replace l with 2
    code = re.sub(r'l', r'2', code)

    # remove all 2's
    code = re.sub(r'2', r'', code)

    # replace the final 3 -> A
    code = re.sub(r'3$', r'A', code)

    # remove all 3's
    code = re.sub(r'3', r'', code)

    # extend the code by 10 '1' (one)
    code += '1' * 10

    # take the first 10 characters
    caverphoneCode = code[:10]

    # return caverphone code
    return caverphoneCode




#!/usr/bin/python
# -*- coding: utf-8 -*-
'''
Created on 23.10.2014

.. codeauthor:: Heinz-Peter Lang <lang@weblyzard.com>

.. seealso:: 

    `Translate API - Getting Started <https://cloud.google.com/translate/v2/getting_started>`_

    `Google Developer Console <https://console.developers.google.com>`_
        Configure your application in the developer console
    
'''
import json
import requests
import logging

from six import string_types
from urllib import urlencode

from eWRT.ws import AbstractWebSource


API_URL = 'https://www.googleapis.com/language/translate/v2'


class GoogleTranslator(AbstractWebSource):
    NAME = 'google_translate'
    SUPPORTED_PARAMS = ('text', 'target_language', 'source_language')
    logger = logging.getLogger('google_translator')

    def __init__(self, api_key, api_url=API_URL):
        self.api_key = api_key
        self.api_url = api_url

    def search_documents(self, search_terms, source_language, target_language):
        ''' translates the `search_terms` from `source_language to the 
        `target_language`
        :returns: iterator with the translated text
        '''
        if isinstance(search_terms, string_types):
            search_terms = [search_terms]

        for search_term in search_terms:
            self.logger.info('... will translate "%s" to %s',
                             search_term,
                             target_language)
            result = self.translate(text=search_term,
                                    target_language=target_language,
                                    source_language=source_language)

            translations = []

            if 'error' in result:
                self.logger.error(result['error'])
                yield result
                continue

            for t in result['data']['translations']:
                lang_key = 'detectedSourceLanguage'
                source_lang = t[lang_key] if lang_key in t else source_language
                translations.append((t['translatedText'],
                                     source_lang,
                                     target_language))

            yield {'text': search_term,
                   'translations': translations}

    def _make_request(self, params, path=''):
        ''' makes the request to GoogleAPI '''
        if not 'key' in params:
            params['key'] = self.api_key
        full_url = self.api_url + path + '?' + urlencode(params)
        resp = requests.get(full_url)
        return json.loads(resp.text)

    def translate(self, text, target_language, source_language=None):
        ''' translates the text '''
        if isinstance(text, unicode):
            text = text.encode('utf8')
        params = {'target': target_language, 'q': text}

        if source_language:
            params['source'] = source_language

        return self._make_request(params)

    def detect_language(self, text):
        ''' detects the language of the given `text` '''
        return self._make_request({'q': text}, path='/detect')



#!/usr/bin/env python

""" @package eWRT.ws.wikipedia
    provides access to wikipedia using the wikipedia api
    http://en.wikipedia.org/w/api.php 
"""

# (C)opyrights 2008 by Albert Weichselbraun <albert@weichselbraun.net>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__version__ = "$Header$"

import unittest

from urllib import urlencode
from xml.dom.minidom import parseString
from nose.plugins.attrib import attr

from eWRT.access.http import Retrieve

WIKIPEDIA_API_QUERY = 'http://%s.wikipedia.org/w/api.php'

class WikiPedia(object):
    """ returns an WikiPedia Object  """

    def __init__(self):
        self.r = Retrieve( WikiPedia.__name__ )
    
    def getWikiPage(self, pageName, lang='en'):
        """ returns the given wikipedia page considering different spellings 
            @param[in] pageName
            @param[in] language (determines which wikipedia to query)
            @returns the page's wikipedia text
        """
        assert( len(lang)==2 )

        for pn in self._getPageNameAlterations( pageName ):
            pageContent = self._retrievePage( pn, lang )
            if pageContent:
                return pageContent

        return None

    @staticmethod
    def _getPageNameAlterations(pageName):
        """ @returns a list of differnt names for the given page """

        alt = [ pageName, ]
        if not ' ' in pageName:
            alt

        words = pageName.split(" ")
        alt.append( "%s %s" % (words[0].capitalize(), " ".join( map(str.lower, words[1:] ) )) )
        return alt

    def _retrievePage(self, pageName, lang):
        """ retrieves the given Wiki page
            @param[in] pageName
            @param[in] language (determines which wikipedia to query)
            @returns the page's wikipedia text
        """
        param = urlencode( {'action': 'query',
                            'format':'json', 
                            'export':'',
                            'redirects':'true',
                            'titles':pageName 
        })
        data = self.r.open( WIKIPEDIA_API_QUERY % lang, param ).read()
        jsonData = eval( data  )['query']
        if '-1' in jsonData['pages']:
            return None

        xmlData = jsonData['export']['*'].replace("\/","/")
        return parseString( xmlData  ).getElementsByTagName('text')[0].firstChild.data



class CleanupWikiText(object):
    """ cleans wiki text """

    @staticmethod
    def removeLanguageReferences(text):
        cleaned = []
        for line in text.split("\n"):
            if line.startswith("<!--") and "Other languages" in line:
                break
            cleaned.append(line)

        return "\n".join(cleaned)
        

class TestWikiPedia(unittest.TestCase):
    """ tests the WikiPedia Class """
    TEST_QUERIES= { 
                ('Energy', 'en'):  ('fossil', 'renewable'),
                ('Energie', 'de'): ('kinetisch', 'Verbrauch', "Atom"),
                 }

    def setUp(self):
        self.w = WikiPedia()

    @attr("remote")
    def testRetrievePage(self):
        """ tries to retrieve the following url's from the list """

        for (keyword, lang), searchTerms in self.TEST_QUERIES.iteritems():
            wikiPediaText = self.w.getWikiPage(keyword, lang=lang)
            for term in searchTerms:
                print keyword, term
                assert term in wikiPediaText

    @attr("remote")
    def testAlternations(self):

        print self.w._getPageNameAlterations("Greenhouse Gas Emissions")
        assert self.w._getPageNameAlterations("Greenhouse Gas Emissions") == ['Greenhouse Gas Emissions', 'Greenhouse gas emissions']
        
    @attr("remote")
    def test_removeLanguageReferences(self):
        
        text = self.w.getWikiPage('Energy', 'en')

        otherLanguages = ['fi:Energia', 'sl:Energija]', 'mwl:Einergie']

        cleantText = CleanupWikiText.removeLanguageReferences( text )

        for term in otherLanguages:
            
            assert term not in cleantText

if __name__ == '__main__':
    
    unittest.main()



#!/usr/bin/env python

""" @package eWRT.util.exception """
from eWRT.config import SNMP_MODULE_NAME, SNMP_LOG_LEVEL, SNMP_HOST_CFG

# (C)opyrights 2008-2010 by Albert Weichselbraun <albert@weichselbraun.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Albert Weichselbraun"
__revision__ = "$Id$"
__copyright__ = "GPL"

from subprocess import call


global MODULE
MODULE = SNMP_MODULE_NAME


class SNMPException(Exception):
    """ reports an exception to snmp """

    def __init__(self, module_name, msg, level="warning"):

        assert module_name in MODULE
        assert level in SNMP_LOG_LEVEL
        assert "\n" not in msg

        self.module_name = module_name
        self.msg = msg
        self.level = level
        self._sendSNMPNotification(module_name, msg, level)

    def _sendSNMPNotification(self, module_name, msg, level):
        """
        sends a copy of the exception to the snmp server
        @param[in] module_name ... name of the module throwing the exception
        @param[in] msg         ... the exception message string
        @param[in] level       ... level of the log message (ok, warning, critical)
        """
        cmd = ["snmptrap", "-v", "2c",
                           "-c", SNMP_HOST_CFG['community_string'],
                           SNMP_HOST_CFG['server'], "''",
                           SNMP_HOST_CFG['oid'],
                           SNMP_MODULE_NAME[module_name], "s", msg,
                           SNMP_MODULE_NAME['webLyzard.log.level'], "i", SNMP_LOG_LEVEL[level]]
        call(cmd)

    def __str__(self):
        return "Exception in '%s': %s" % (self.module_name, self.msg)


class TestSNMPException(object):

    def testSNMPException(self):
        """ tests an SNMP exception """
        try:
            raise SNMPException("webLyzard.experimental",
                                "This is a test exception")
        except SNMPException:
            pass

    def testQuoting(self):
        try:
            raise SNMPException("webLyzard.experimental", "Hallo wie geht's?")
        except SNMPException:
            pass

#t = TestSNMPException()
# t.testSNMPException()
# $Id$



#!/usr/bin/env python

from __future__ import print_function
from __future__ import absolute_import

# create a iterable ResultSet for storing values of the Webservices


class ResultSet(object):

    # Constructor
    # @parameter id (optional) integer
    # @parameter name (optional) String
    # @parameter content (optional) a Result or ResultSet
    def __init__(self, id=None, name=None, content=None):
        self.id = id
        self.name = name
        self.setContent(content)
        self.idx = 0

    # returns the next item in the ResultSet
    # @result or ResultSet
    # @return content = Result, ResultSet
    def next(self):
        # if self.idx > self.content.count:
        if self.idx >= len(self.content):
            self.refresh()
            raise StopIteration
        result = self.content[self.idx]
        self.idx += 1
        return result
    # function for the iterator

    def __iter__(self):
        return self

    # sets the pointer of the index to 0
    def refresh(self):
        self.idx = 0

    # sets the content to the new content
    # overrides the content
    # @parameter content = Result, ResultSet
    def setContent(self, content):
        if content is None:
            self.content = []
        else:
            self.content = content

    # adds new content to the existing one
    # @parameter newContent = Result or ResultSet
    def addContent(self, newContent):
        self.content.append(newContent)
        """ todo: needs to be implemented """

    # return the ID of the ResultSet
    # @return ID integer
    def getId(self):
        return self.id

    # return the Name of the ResultSet
    # @return Name String
    def getName(self):
        return self.name

    # prints the items stored in the ResultSet
    @staticmethod
    def printRS(resultSet, filler=0):
        print(resultSet.getName(), ':')

        for result in resultSet:
            if cmp(result.__class__.__name__, 'ResultSet') == 0:
                # filler = filler + 1
                ResultSet.printRS(result, filler)
            else:
                print(result.getId(), result.getName())


if __name__ == "__main__":

    from .Result import Result

    a = Result(1, 'foobar1')
    b = Result(2, 'foobar2')
    c = Result(3, 'foobar3')
    d = Result(4, 'foobar4')
    e = Result(5, 'foobar5')
    f = Result(6, 'foobar6')
    g = Result(7, 'foobar7')
    h = Result(8, 'foobar8')
    i = Result(9, 'foobar9')

    content = [a, b, c, d]
    rs = ResultSet(1, 'RS1')
    rs.addContent(a)
    rs.addContent(b)
    rs.addContent(c)
    rs.addContent(d)

    rs2 = ResultSet(1, 'RS2')
    rs2.addContent(rs)
    rs2.addContent(e)
    rs2.addContent(f)

    rs2.printRS(rs2)



""" @package eWRT.ontology.eval
    Evaluates ontologies based on their _internal_ characteristics such as 
    term coherence, internal integrity, ...

    For evaluations against a reference ontology
    @see eWRT.ontology.compare
"""



# #!/usr/bin/env python
# 
# import re
# 
# from eWRT.access.http import Retrieve
# from eWRT.ws.TagInfoService import TagInfoService
# 
# 
# class Twitter(TagInfoService):
# 
#     TWITTER_SEARCH_URL = 'http://search.twitter.com/search.json?q=&tag=%s&lang=all&rpp=100'
#     RE_FIND_TAGS = re.compile('#(\w+)', re.IGNORECASE|re.DOTALL)
# 
#     @staticmethod
#     def getRelatedTags( tags ):
#         """ fetches the related tags for the given tags
#             @param list of tags
#             @return dictionary of related tags with count
#         """
# 
#         if type(tags).__name__ == 'str':
#             url = Twitter.TWITTER_SEARCH_URL % tags
#         else:   
#             url = Twitter.TWITTER_SEARCH_URL % "+".join(tags)
# 
#         f = Retrieve(Twitter.__name__).open(url)
# 
#         # convert json into dict and remove null values with ""
#         search_results = eval(re.sub('null', '""', f.read()))
#         found_tags = []
#         related_tags = {}
# 
#         for result in search_results['results']:
#             found_tags.extend(Twitter.RE_FIND_TAGS.findall( result['text']))
# 
#         for tag in found_tags:
#             related_tags[tag.lower()] = related_tags.get(tag.lower(), 0) + 1
# 
#         # todo: sort
# 
#         return related_tags
#     
#     @staticmethod
#     def getDailyTrends():
#         raise NotImplementedError




#!/usr/bin/env python

""" @package eWRT.ws.flickr
    retrieve flickr related tags and tag counts.    
"""

# (C)opyrights 2008-2010 by Albert Weichselbraun <albert@weichselbraun.net>
#                           Heinz-Peter Lang <heinz@langatium.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


__version__ = "$Header$"

import re
from eWRT.access.http import Retrieve
from eWRT.ws.TagInfoService import TagInfoService


class Flickr(TagInfoService):
    """ retrieves data using the del.icio.us API """

    FLICKR_TAG_URL = "http://www.flickr.com/photos/tags/%s"
    RE_TAG_COUNT = re.compile('<div class="Results">\(([\d,]+) upload')
    RE_TAG_CONTAINER = re.compile(
        '<p>Related tags:<br>(.*?)</p>', re.IGNORECASE | re.DOTALL)
    RE_RELATED_TAGS = re.compile('<a.*?">(.*?)</a>', re.IGNORECASE | re.DOTALL)

    __slots__ = ()

    @staticmethod
    def getTagInfo(tags):
        """ returns the number of counts for the given tags
            @param   tags   A list of tags to retrieve information for
            @returns        the number of bookmarks using the given tags
        """
        assert isinstance(tags, list) or isinstance(tags, tuple)

        url = Flickr.FLICKR_TAG_URL % "+".join(tags)
        content = Flickr.get_content(url)
        return Flickr._parse_tag_counts(content)

    @staticmethod
    def getRelatedTags(tags, retrieveTagInfo=False):
        """ fetches the related tags with their overall count
            @param  tags             list of tags
            @param  retrieveTagInfo  determines whether we will retrieve the tagInfo for the related tags
            @returns                 list of related tags 
        """
        assert isinstance(tags, list) or isinstance(tags, tuple)

        url = Flickr.FLICKR_TAG_URL % "+".join(tags)
        content = Flickr.get_content(url)
        tag_container = Flickr.RE_TAG_CONTAINER.findall(content)
        related_tags_with_count = []

        if len(tag_container) > 0:
            related_tags = re.sub('</?b>', '', tag_container[0])
            related_tags = Flickr.RE_RELATED_TAGS.findall(related_tags)

            related_tags_with_count = [(tag, Flickr.getTagInfo(
                (tag,)) if retrieveTagInfo else None) for tag in related_tags]

        return related_tags_with_count

    #
    # helper functions
    #
    @staticmethod
    def _parse_tag_counts(content):
        """ parses flickrs html content and returns the number of counts for the tags """
        m = Flickr.RE_TAG_COUNT.search(content)
        return int(m.group(1).replace(",", "")) if m else 0

    @staticmethod
    def get_content(url):
        """ returns the content from Flickr """
        assert(url.startswith("http"))

        f = Retrieve(Flickr.__name__).open(url)
        content = f.read()
        f.close()
        return content



#!/usr/bin/python
# -*- coding: utf-8 -*-
'''
Created on Dec 13, 2011

:author: heinz
'''
import json
import logging

from urllib import urlencode
from datetime import datetime

from eWRT.ws.WebDataSource import WebDataSource
from eWRT.access.http import Retrieve


API_URL = 'https://www.googleapis.com/plus/v1/{path}?{query}'
DEFAULT_ORDER_BY = 'recent'  # other possibility: best
MAX_RESULTS_PER_PAGE = 20
DEFAULT_MAX_RESULTS = 20  # requires only 1 api access

logger = logging.getLogger('eWRT.ws.google')


class GooglePlus(object):
    '''
    classdocs
    '''

    def __init__(self, api_key, api_url=API_URL):
        ''' Constructor      '''
        WebDataSource.__init__(self)
        self.api_key = api_key
        self.api_url = api_url
        self.retrieve = Retrieve('google-plus')

    def search(self, search_terms, max_results=DEFAULT_MAX_RESULTS):
        ''' searches Google+ for the given search_terms 
        :param search_terms: search terms
        :type search_terms: list
        :param max_results: maximum number of result
        :type max_results: int
        :returns: generator with the result
        '''
        for search_term in search_terms:
            if isinstance(search_term, unicode):
                search_term = search_term.encode('utf-8')
            params = {'query': '"%s"' % search_term,
                      'orderBy': DEFAULT_ORDER_BY,
                      'maxResults': max_results}

            fetched = self.make_request(params, 'activities')

            for item in fetched['items']:
                try:
                    yield self.convert_item(item)
                except Exception as e:
                    logger.info('Error %s occured' % e)
                    continue

    def get_activity(self, activity_id):
        ''' returns the activity with the given ID
        :param activity_id: GooglePlus activity ID
        :type activity_id: string
        :returns: mapped result
        :rtype: dict
        '''
        item = self.make_request(path='activities/%s' % activity_id)
        return self.convert_item(item)

    def make_request(self, params=None, path='activities'):
        ''' executes the request to GooglePlus
        :param params: paremeters for the query
        :type params: list or None
        :param path: path to query, e.g. activities
        :type path: string
        :returns: GooglePlus result
        :rtype: dict
        '''
        url = self.get_request_url(params, path)
        data = self.retrieve.open(url)
        return json.load(data)

    def get_request_url(self, params=None, path='activities'):
        ''' returns a correctly parsed request URL 
        :param params: paremeters for the query
        :type params: list or None
        :param path: path to query, e.g. activities
        :type path: string
        :returns: GooglePlus request URL
        :rtype: str

        Usage: 
            >>> plus = GooglePlus('abcd')
            >>> plus.get_request_url()
            'https://www.googleapis.com/plus/v1/activities?key=abcd'
        '''
        params = params if params else {}

        if not 'key' in params:
            params['key'] = self.api_key

        if 'maxResults' in params and params['maxResults'] > DEFAULT_MAX_RESULTS:
            params['maxResults'] = DEFAULT_MAX_RESULTS

        return self.api_url.format(path=path, query=urlencode(params))

    @classmethod
    def convert_item(cls, item):
        ''' applies a mapping to convert the result to the required format
        :param item: GooglePlus Activity
        :type item: dict
        :rtype: dict
        '''

        last_modified = datetime.strptime(item['updated'],
                                          '%Y-%m-%dT%H:%M:%S.%fZ')
        published = datetime.strptime(item['updated'],
                                      '%Y-%m-%dT%H:%M:%S.%fZ')

        content = cls.convert_content(item['object']['content'])

        if not item['verb'] == 'post':
            raise Exception('Skipping activity of type "%s"' % item['verb'])

        if not len(content):
            logger.info('Skipping "%s" -> content is empty' % item['title'])
            raise Exception('content is empty')

        if 'attachments' in item['object']:
            for attachment in item['object']['attachments']:
                if attachment['objectType'] == 'article':
                    if not 'content' in attachment:
                        raise Exception('no content in attachment')

                    content = '%s\n"%s" (%s)' % (content,
                                                 cls.convert_content(
                                                     attachment['content']),
                                                 attachment['url'])

        activity = {'content': content,
                    'title': item['actor']['displayName'],
                    'url': item['url'],
                    'last_modified': last_modified,
                    'user_id': item['actor']['id'],
                    'user_img_url': item['actor']['image']['url'],
                    'screen_name': item['actor']['displayName'],
                    'encoding': u'utf-8',
                    'user_url': item['actor']['url'],
                    'valid_from': published,
                    'reshares': item['object']['resharers']['totalItems'],
                    'plusoners': item['object']['plusoners']['totalItems'],
                    'activity_id': item['id'],
                    }

        if 'geocode' in activity:
            activity['geocode'] = item['geocode']

        return activity


if __name__ == '__main__':
    import doctest
    doctest.testmod()



__author__ = 'Philipp Konrad'


import urllib
import requests
import json
import pycurl
import os

TEST_JSON = '''
{
  "column_1":"You say goodbye",
  "column_2":"And I say hello",
  "column_3":"Hello, hello"
}
{
  "column_1":"I don't know why you say goodbye",
  "column_3":"I say hello"
}
'''

class CrowdFlowerError(Exception): pass

'''
{u'alias': None,
 u'auto_order': False,
 u'auto_order_threshold': None,
 u'auto_order_timeout': None,
 u'cml': None,
 u'completed': False,
 u'completed_at': None,
 u'confidence_fields': None,
 u'created_at': u'2014-02-27T10:25:02+00:00',
 u'css': None,
 u'custom_key': None,
 u'design_verified': False,
 u'desired_requirements': None,
 u'excluded_countries': [],
 u'execution_mode': u'worker_ui_remix',
 u'expected_judgments_per_unit': None,
 u'fields': None,
 u'gold': {},
 u'gold_per_assignment': 0,
 u'golds_count': 0,
 u'id': 393918,
 u'included_countries': [],
 u'instructions': u'Good night and good fight',
 u'js': None,
 u'judgments_count': 0,
 u'judgments_per_unit': 3,
 u'language': u'en',
 u'max_judgments_per_ip': None,
 u'max_judgments_per_unit': None,
 u'max_judgments_per_worker': None,
 u'min_unit_confidence': None,
 u'minimum_account_age_seconds': None,
 u'minimum_requirements': {u'min_score': 1,
                           u'priority': 1,
                           u'skill_scores': {u'level_1_contributors': 1}},
 u'options': {u'after_gold': 4,
              u'front_load': False,
              u'ramuh2_url': u'http://ramuh2-003.cloud.doloreslabs.com:8080/ramuh/api/v2',
              u'track_clones': True},
 u'order_approved': False,
 u'pages_per_assignment': 1,
 u'problem': None,
 u'project_number': None,
 u'public_data': False,
 u'require_worker_login': None,
 u'send_judgments_webhook': None,
 u'state': u'unordered',
 u'support_email': u'selfservice_notifications@crowdflower.com',
 u'title': u'Another upload',
 u'units_count': 0,
 u'units_per_assignment': 5,
 u'units_remain_finalized': None,
 u'updated_at': u'2014-02-27T10:25:02+00:00',
 u'variable_judgments_mode': u'none',
 u'webhook_uri': None,
 u'worker_ui_remix': True}
 '''


class CrowdFlowerClient(object):
    '''
    Crowdflower Client for Crowdflower API Version 1

        >>> api_key = 'your_api_key'
        >>> client = CrowdflowerClient(api_key)
    '''
    CROWDFLOWER_BASIC_URL = 'https://api.crowdflower.com/v1/'
    JOB_URL_PTN = 'jobs/upload.json?key=%(api_key)s'


    def __init__(self, api_key):
        '''
        :param str api_key: An API key taken from https://crowdflower.com/account
        '''
        self.query_params = {'api_key':api_key}
        self.api_key = api_key

    def post_job(self, job):
        '''
        :param job:
        '''

        assert 'title' in job, 'Job title is missing %s' % job
        assert 'instructions' in job, 'Job instructions are missing %s' % job

        headers = {'content-type': 'application/json'}

        args = {'job[%s]' % k if k != 'key' else k:v for k, v in job.iteritems()}
        qry_args = urllib.urlencode(args)

        url = "https://api.crowdflower.com/v1/jobs/%s.json?" % job['id']
        url = url + qry_args

        response = requests.put(url, data=json.dumps(job), headers=headers)
        return response.json()


    def create_job(self):
        '''
        :param str title:
        :param str instructions:
        :param int num_judgements:
        :param int cost_in_cent:
        '''
        job_url = self.JOB_URL_PTN % self.query_params
        headers = {'content-type': 'application/json'}
        url = self.CROWDFLOWER_BASIC_URL + job_url

        response = requests.post(url, headers=headers)
        id = response.json().get('id')

        if not id: raise CrowdFlowerError("Could not create job!")

        return CrowdFlowerJob(self, self.api_key, id)



class CrowdFlowerJob(dict):
    '''
    A job helps you to define all properties of a wished job
    and synchronize it with Crowdflower.

        >>> api_key = 'your_api_key'
        >>> client = CrowdflowerClient(api_key)
        >>> job = client.create_job(job_id=1234)

        >>> job.sync()

    '''

    def __init__(self, client, api_key, id):
        super(CrowdFlowerJob, self).__init__()
        self.client = client
        self['id'] = id
        self['key'] = api_key

    def sync(self):
        '''
        Synchronize job with Crowdflower.
        :returns:
        '''
        return self.client.post_job(self)

class CrowdFlowerClient2(object):

    def __init__(self, key):

        self.key = key
        self.upload_url_pattern = 'http://api.crowdflower.com/v1/jobs/%s/upload.json?key=' + key


    def create_job(self, title, instructions, cml=None, callback=None):

        url = "http://api.crowdflower.com/v1/jobs.json"

        post_dict = {'key':self.key,
                     'job[title]':title,
                     'job[instructions]':instructions}

        if cml: post_dict['job[problem]'] = cml

        post_fields = urllib.urlencode(post_dict)

        c = pycurl.Curl()
        c.setopt(pycurl.POST, True)
        c.setopt(pycurl.URL, url)
        c.setopt(pycurl.POSTFIELDS, post_fields)

        if callback: c.setopt(pycurl.WRITEFUNCTION, callback)

        c.setopt(pycurl.HTTPHEADER, ['Content-Type : application/json', ])
        c.perform()
        c.close()


    def upload_csv(self, fname, job_id):

        upload_url = self.upload_url_pattern % job_id

        c = pycurl.Curl()
        c.setopt(pycurl.URL, upload_url)
        c.setopt(pycurl.UPLOAD, True)

        c.setopt(pycurl.READFUNCTION, open(fname, 'r').read)

        filesize = os.path.getsize(fname)

        c.setopt(pycurl.INFILESIZE, filesize)
        c.setopt(pycurl.HTTPHEADER, ['Content-type: text/csv'])

        c.perform()
        c.close()


    def create_and_upload(self, title, instructions, fname, cml):

        def job_created_callback(response):

            response_dict = json.loads(response)
            job_id = response_dict['id']
            self.upload_csv(fname=fname, job_id=job_id)

        self.create_job(title, instructions, cml, job_created_callback)


if __name__ == '__main__':
    cml= open('/tmp/test.cml').read()
    
    
    client_a = CrowdFlowerClient('pZMDXJHB9J6cCzVV7ctu')
    client_b = CrowdFlowerClient2('pZMDXJHB9J6cCzVV7ctu')
    #client.create_and_upload('Finally a CSV','this is created as a test', '/tmp/test.csv', cml)
    
    #
    #client.create_job('Matyas a CSV doit3','this is created as a test', cml)
    client_b.upload_csv('/tmp/test.csv', 393963)



#!/usr/bin/env python

# Result is an item of ResultSet


class Result(object):

    # constructor
    # @parameter id, name
    def __init__(self, id, name):
        self.id = id
        self.name = name

    # get the ID of Result
    # @return Id
    def getId(self):
        return self.id

    # get the Name of the Result
    # @return Name
    def getName(self):
        return self.name

    def getAttributes(self):
        """ todo: return attributes of Result """



#!/usr/bin/env python
"""
 @package eWRT.ontology.compare.relationtypes
"""

# (C)opyrights 2010 by Albert Weichselbraun <albert@weichselbraun.net>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from nltk import WordNetLemmatizer, pos_tag
from collections import defaultdict

from eWRT.util.cache import MemoryCache, MemoryCached


class RelationTypes(object):

    SHORT_POS = { 
        'IN': 'p', 'TO': 'p', 'RB': 'p', 'RP': 'p',
        'VB': 'v', 'MD': 'v', 
        'NN': 'v'                   # map nouns to verbs as they are the default results
    }

    AUXILIARY_VERBS = ('be', 'have', 'may', 'should', 'can', 'do', 'might', 'must' 'ought', 'shall', 'would')
    
    def __init__(self):
        self.w = WordNetLemmatizer()
        self.cache = MemoryCache()
        # self.cache.fetchObjectId(text,  self.w.lemmatize, text.split(), pos='v' ):

    @staticmethod
    @MemoryCached
    def getPos(text):
        """ returns the pos tags for the given text """
        posMap = []
        for term, pos in pos_tag( text.split() ):
            mapping = [ spos for lpos, spos in RelationTypes.SHORT_POS.items() if pos.startswith(lpos) ]
            pos = mapping[0] if mapping else ''
            posMap.append( pos )

        return posMap


    def partitionRelation(self, text, removeAuxiliaryVerbs=True) :
        """ partitions a relation into the following comparision format:
              {'v': ( v1, ...), 'p': ('by', ...), '': ('also', ...) }
            @param[in] text the text describing the link
            @returns   the partitioned text 
        """
        result = defaultdict( list )
        for term, pos in zip( text.split(), self.getPos(text)):
            result[pos].append(  
                 self.cache.fetchObjectId(term, self.w.lemmatize, term, pos='v' ) if pos=='v' else term )

        if 'v' in result and removeAuxiliaryVerbs:
            result['v'] = self.removeAuxiliaryVerbs( result['v'] )
 
        return result

    @staticmethod
    def removeAuxiliaryVerbs( verbList ):
        """ removes auxiliary verbs from the given list, provided that
            at least one verb remains in the list.

            @param[in] verbList a list of verbs to analyze
            @returns   the cleaned verblist 
        """
        return [ v for v in verbList if v not in RelationTypes.AUXILIARY_VERBS ] or [ verbList[0] ]


class TestRelationTypes(object):

    def __init__(self):
        self.rt = RelationTypes()

    def testPosTagging(self):
        """ tests the pos tagging """
        print self.rt.getPos("can be saved by")
        assert self.rt.getPos("can be saved by") == ['v', 'v', 'v', 'p']

        print self.rt.getPos("involves")
        assert self.rt.getPos("involves") == ['v', ]

    def testPartitioning(self):
        """ tests the verb partitioning """
        print self.rt.partitionRelation("can be saved by") 
        assert dict(self.rt.partitionRelation("can be saved by")) == {'v': ['save', ], 'p': ['by',],  } 
        assert dict(self.rt.partitionRelation("results in"))      == {'v': ['result', ], 'p': ['in',],  }
        assert dict(self.rt.partitionRelation("e.g."))            == {'v': ["e.g."] }
        assert dict(self.rt.partitionRelation("????"))            == {'v': ["????"] }


    def testRemoveAuxiliaryVerbs(self):
        """ tests the removal of auxiliary verbs """
        assert self.rt.removeAuxiliaryVerbs( ("can", "be", "save", ) ) == ["save", ]

        print self.rt.removeAuxiliaryVerbs( ("be",))
        assert self.rt.removeAuxiliaryVerbs( ("be",)) == ["be",]
        assert self.rt.removeAuxiliaryVerbs( ("can", "range", "high")) == ["range", "high"]
        
